#ifndef _vteGUI_h
#define _vteGUI_h

#include "vteObject.h"
#include "vteDataTypes.h"
#include "vteCollection.h"
#include "vteVisualizationWindow.h"
#include "vteNetwork.h"

#include "QVTKWidget.h"

#include <QMainWindow>
#include <QMdiArea>
#include <QToolbar>
#include <QMenuBar>
#include <QAction>
#include <QMenu>
#include <QTimer>
#include <QCloseEvent>

class vteGUIWindow;

//*********************
class vteGUI : public QMainWindow, public vteObject
{
    Q_OBJECT

public:
    vteGUI( );
    ~vteGUI();

   vteGUIWindow *GetActiveWindow() {return active_window;}
   void SetActiveWindow(vteGUIWindow *window) {active_window=window;}

public slots:
    virtual void ToggleScalarGradient();
    virtual void ToggleGeometry();
    virtual void ToggleGrid();
    virtual void ToggleGlyphs();
    virtual void TogglePlanes();
    virtual void ToggleIsosurface();
    virtual void TogglePlay();
    virtual void ToggleRecord();
    virtual void ToggleNetwork();
    virtual void ToggleRedBlue();
    virtual void ToggleCrystalEyes();
    virtual void ToggleNormalView();
    virtual void OpenDataset();
    virtual void SaveDataset();
    virtual void Close();
    virtual void GetGlyphProperties();
    virtual void GetIsoProperties();
    virtual void GetPlaneProperties();
    virtual void GetGridProperties();
    virtual void GetPlaybackProperties();
    virtual void ShowHelp();
    virtual void About();
    virtual void ActiveWindowChanged(QMdiSubWindow *);
    virtual void DisableToolbar();

protected:
    vteGUIWindow *active_window;
	bool toggle_action_disabled;

    QMdiArea *workspace;
    QMenuBar *menubar;
    QMenu *fileMenu;
    QMenu *ViewMenu;
    QMenu *SettingsMenu;
    QMenu *PlaybackMenu;
    QMenu *NetworkMenu;
    QMenu *HelpMenu;
    QToolBar *toolbarFile;
    QToolBar *toolbarToggle;
    QToolBar *toolbarPlay;
    QToolBar *toolbarView;
    QToolBar *toolbarNetwork;
    QAction* actionOpen;
    QAction* actionSave;
    QAction* actionClose;
    QAction* actionExit;
    QAction* actionToggleGeometry;
    QAction* actionToggleGlyphs;
    QAction* actionToggleGrid;
    QAction* actionTogglePlanes;
    QAction* actionToggleIsosurface;
    QAction* actionToggleScalarGradient;
    QAction* actionIsoProperties;
	QAction* actionGlyphProperties;
    QAction* actionGridProperties;
    QAction* actionPlanesProperties;
    QAction* actionPlaybackProperties;
    QAction* actionTogglePlay;
    QAction* actionToggleRecord;
    QAction* actionToggleNetwork;
    QAction* actionToggleRedBlue;
    QAction* actionToggleCrystalEyes;
    QAction* actionToggleNormalView;

    QAction* actionHelp;
    QAction* actionAbout;

protected slots:
    virtual void languageChange();
};

#endif