#ifndef _vteGUIWindow_h
#define _vteGUIWindow_h

#include <QMdiSubWindow>
#include <QStatusBar>
#include <QTimer>
#include <QString>
#include <QMdiArea>
#include <QCloseEvent>

#include <vtkVectorText.h>
#include <vtkPolyDataMapper.h>
#include <vtkFollower.h>

#include "vteObject.h"
#include "vteGUI.h"

#include "vteNetwork.h"
#include "vteVisualizationWindow.h"
#include "vteGeometry.h"
//----------

class vteNetwork;
class vteGUI;

struct vteNetworkClientInfo
  {
    int id;
    QString name;
    QString avatar;
    vtkVectorText *source;
    vtkPolyDataMapper *mapper;
    vtkFollower *follower;
  };

class vteGUIWindow: public QMdiSubWindow, public vteObject
{
    Q_OBJECT
public:
	vteGUIWindow( QMdiArea* workspace, vteGUI *gui, QStatusBar *status_bar, 
		const char* file_name, Qt::WindowFlags flags);
    ~vteGUIWindow();

	vteVisualizationWindow *GetVisualizationWindow() {return qvtk_window;}
	
	bool IsPlaying() {return playing;}
    bool IsRecording() {return recording;}
    bool GetIsoPropertiesAuto() {return iso_properties_auto;}
    void SetIsoPropertiesAuto(bool flag) {iso_properties_auto=flag;}

    int GetPlaybackRate() {return playback_rate;}
	void SetPlaybackRate(int rate);

    int TogglePlay();
    int ToggleRecord();

    vteNetwork *GetNetwork() {return network;}
	vteNetworkStatus *GetNetworkStatus() {return network_status;}
    int StartNetwork(long port, const QString &name, const QString &avatar, bool server, const QString &address);
    void StopNetwork();
    bool IsNetworkConnected() {return network_connected;}
    vteNetworkClientInfo *GetNetworkClientInfo() {return network_client_info;}
    int GetNumberOfClients() {return number_of_clients;}

signals:
    void message(const QString&, int );
		void closeSignal();

public slots:
    void NetworkConnectionStatus(int flag);

private slots:
	void PlaybackTimerOverhead();
    void RecordTimerOverhead();
    void ProcessNetworkData(QString & data);
    void NetworkTimer();

protected:
	void closeEvent (QCloseEvent *e){emit closeSignal();e->accept();}
    void AddAvatar(const char *name);

protected:
    QTimer *playback_timer;
    QTimer *record_timer;
    QTimer *network_timer;
    
    QStatusBar *status_bar;
    
	vteNetwork *network;
	vteNetworkStatus *network_status;

    vteGUI *parent_gui;
	
	vteVisualizationWindow *qvtk_window;
    QVTKWidget *qvtk_widget;

    vteGeometryCollection *avatar_collection;
    vteNetworkClientInfo network_client_info[MAX_CLIENTS];
    int number_of_clients;
    
    bool playing;
    bool recording;
	bool iso_properties_auto;
    bool network_connected;

	int playback_rate;
    int record_name_index;
};

#endif
