#ifndef _vteGeometryCollection_h
#define _vteGeometryCollection_h

#include "vteStorageClass.h"
#include "vteCollection.h"
#include "vteDataTypes.h"
#include "vteGeometry.h"

#include "vteCollection.h"
#include "vteDataTypes.h"
#include "vteStorageClass.h"
#include "vteGrid.h"
#include "vtkPolyData.h"
#include "vtkPolyDataMapper.h"
#include "vtkLookupTable.h"
#include "vtkLODActor.h"

class vteGeometryForVtk;

class vteGeometry:public vteStorageClass
{
public:
	vteGeometry();
	~vteGeometry();

	int AddColor (vteColor color);
	int AddNodes (vteNode *nodes, int number_of_nodes);
	int AddCells (vteCell *cells, int number_of_cells);

	unsigned long GetNumberOfNodes() {return number_of_nodes;}
	unsigned long GetNumberOfCells() {return number_of_cells;}
	unsigned long GetNumberOfColors() {return number_of_colors;}

	vteNode* GetNodes() {return nodes;}
	vteCell* GetCells() {return cells;}
	vteColor* GetColors() {return colors;}

	vteGeometryForVtk* GetGeometryForVtk();

private:
	unsigned long number_of_nodes;
	unsigned long number_of_cells;
	unsigned int number_of_colors;
	vteNode *nodes;
	vteCell *cells;
	vteColor *colors;

	vteGeometryForVtk *geometry_for_vtk;
};

class vteGeometryCollection:public vteCollection
{                                                                      
public:
	vteGeometryCollection();
	~vteGeometryCollection();
 
	vteGeometry *NewGeometry();
	vteGeometry *GetGeometry(int i);
 
protected: 
	vteGeometry **geometry;
};


class vteGeometryForVtk:public vteStorageClass
{
public:
	static vteGeometryForVtk* New(vteGeometry *geometry,vteGridFrameCollection *grid_frame_collection);
	void Delete();

	vteGeometryForVtk();
	~vteGeometryForVtk();

	vtkLookupTable *GetLut() {return lut;}
	vtkLODActor *GetActor() {return actor;}
	vtkPolyData *GetPolydata() {return polydata;}
	vtkPolyDataMapper *GetMapper() {return mapper;}
	double *GetScalars() {return ptr_to_scalars;}
	double *GetColorScalarsBackup() {return color_scalars_backup;}
	vteBoxIndex *GetBoxIndex() {return box_index;}
	vteGeometry *GetInput() {return input;}
 
protected:
	
	vtkPolyData *polydata;
	vtkPolyDataMapper *mapper;

	vtkLookupTable *lut;
	vtkLODActor *actor;

 	vteBoxIndex *box_index;

	double *ptr_to_nodes;               //pointers for direct memory manipulation
	double *ptr_to_scalars;
	double *color_scalars_backup;

	vteGeometry *input;
};

#endif
