#ifndef _vteGlyphSetCollection_h
#define _vteGlyphSetCollection_h

#include "vteStorageClass.h"
#include "vteCollection.h"
#include "vteDataTypes.h"
#include "vteFrame.h"

#include "vtkPointSource.h"
#include "vtkSphereSource.h"
#include "vtkConeSource.h"
#include "vtkCubeSource.h"
#include "vtkPolyData.h"
#include "vtkStructuredGrid.h"
#include "vtkStructuredGridGeometryFilter.h"
#include "vtkGlyph3D.h"
#include "vtkLODActor.h"
#include "vtkLookupTable.h"
#include "vtkVolume.h"

class vteGlyphFrameCollection;
class vteGlyphFrame;
class vteGlyphSetForVtk;

/***********************************
vteGlyphSetCollection

GlyphSetCollection is a collection of glyphs,
specified by GLYPH_SET command. Each item in the
collection can contain multiple frames, with 
each frame containing variable number of glyphs

vteGlyphSetCollection
 \-> vteGlyphFrameCollection
	  \-> vteGlyphFrame
		   \-> point data
		   \-> VTK structures
*************************************/
class vteGlyphSetCollection:public vteCollection
{
public:
	vteGlyphSetCollection();
	~vteGlyphSetCollection();

	vteGlyphFrameCollection *AddNew();
	vteGlyphFrameCollection *Get(int i);

protected:
	vteGlyphFrameCollection **glyph_frame_collection;
};

/*******************************************
vteGlyphFrameCollection

GlyphFrameCollection is a collection of glyph frames, 
each containing arbitrary number of glyphs
********************************************/
class vteGlyphFrameCollection:public vteFrameCollection
{
public:
	vteGlyphFrameCollection();
	~vteGlyphFrameCollection();

	vteGlyphFrame *AddFrame(int nog);
	vteGlyphFrame *GetFrame(long index = -1);
	unsigned int MembersInFrame (int index = -1);
	unsigned int MaxMembers();

	int AddShape(GlyphSource source, double size, vteColor color);
	int GetNumberOfShapes() {return number_of_shapes;}
	vteGlyphGeometry* GetSource(int i) {return &shape[i];}

	vteGlyphSetForVtk *GetGlyphSetForVtk();

private:
	vteGlyphFrame **frame;
	vteGlyphGeometry *shape;
	unsigned int number_of_shapes;
	vteGlyphSetForVtk *glyph_set_for_vtk;
};
  

/*****************************************
vteGlyphFrame

GlyphFrame contains info about all glyphs 
existing in a particular frame. 
******************************************/
class vteGlyphFrame:public vteCollection
{
public:
	vteGlyphFrame (int number_of_glyphs);
	~vteGlyphFrame ();

	int AddGlyph (vteGlyph *glyph);
	vteGlyph *GetGlyphs() {return glyphs;}

private:
	int max_members;
	vteGlyph *glyphs;
};


/*********************
vteGeneralSource

Data structure used by VTK mapper
**********************/
class vteGeneralSource:public vteStorageClass
{
public:
	vteGeneralSource();
	~vteGeneralSource();

	vtkPointSource *point_source;
	vtkSphereSource *sphere_source;
	vtkConeSource *cone_source;
	vtkCubeSource *cube_source;

	vtkPolyData *output;
};

/****************************
vteGlyphSetForVtk

VTK representation
*****************************/
class vteGlyphSetForVtk:public vteStorageClass
{
public:
	static vteGlyphSetForVtk *New(vteGlyphFrameCollection *glyph_set);
	void Delete();

	vtkGlyph3D *GetGlyph() {return glyph;}
	vtkLODActor *GetActor() {return actor;}
	vtkStructuredGrid *GetGrid() {return grid;}
	vteGeneralSource *GetSource(int i) {return source[i];}

	vteGlyphFrame *GetInput() {return input;}
	vtkStructuredGridGeometryFilter *GetFilter() {return filter;}
 
protected:
	vteGlyphSetForVtk();
	~vteGlyphSetForVtk();

	vteGeneralSource **source;
	unsigned int number_of_sources;
	vtkStructuredGrid *grid;

	vtkStructuredGridGeometryFilter *filter;
	vtkGlyph3D *glyph;
	vtkPolyDataMapper *mapper;
	vtkLODActor *actor;
	vtkLookupTable *lut;
	vteGlyphFrame *input;
};

#endif
