#ifndef _vteNetwork_h
#define _vteNetwork_h

#include <QString>
#include <QTcpSocket>
#include <QTcpServer>
#include <QDialog>
#include <QRadioButton>
#include <QLabel>
#include <QTextEdit>
#include <QLineEdit>
#include <QPushButton>

#include "vteGUI.h"

#define PROTOCOL_VERSION 1.2
#define MAX_CLIENTS 20

#define ACCEPT 0
#define DATA_MISMATCH 1
#define REJECT -1

class QRadioButton;
class QTextEdit;
class QLineEdit;
class QLabel;
class QPushButton;
class vteGUIWindow;

/*vteNetworkStatus----------------------*/
class vteNetworkStatus: public QDialog
{
  Q_OBJECT

public:
 vteNetworkStatus(vteGUIWindow *parent, const char* name=0, bool modal = FALSE, Qt::WFlags fl = 0 );
 ~vteNetworkStatus();

 QTextEdit* textview;
 QLineEdit* command_line;
 QRadioButton* server;
 QLineEdit* other;
 QRadioButton* self;
 QPushButton* apply;
 QPushButton* ok;
 QPushButton* disconnect;

protected slots:
 virtual void languageChange();

private slots:
 void setRemoteView(bool remote=true);
 void executeCommand();
 void setRemoteView2();
 void networkDisconnect();
 //void setLocalView(bool remote);

private:
 QString name;
 vteGUIWindow *parent_window;
};

/* vteClient*/
class vteClient:public QTcpSocket
{
  Q_OBJECT
public:
  vteClient(long port, const QString &address, const QString &name, int speed, vteGUIWindow *parent);
	~vteClient();
  void Send(const QString &string);
  int GetSpeed() {return speed;}

private slots:
  void closeConnection();
  void socketReadyRead();
  void socketConnected();
  void socketConnectionClosed();
  void socketClosed();
  void socketError(QAbstractSocket::SocketError);

signals:
  void ConnectionStatus(int);
  void NewData(QString& data);
  
private:
  long port;
  int id;
  QString name;
  int speed;
  QTcpSocket *socket;
  vteGUIWindow *parent_window;
  bool connected;
};


/*vteClientSocket -- used by server*/
class vteClientSocket:QObject
{
  Q_OBJECT
  public:
  vteClientSocket(vteGUIWindow *parent, QTcpSocket *socket=NULL, const char *name=NULL);
  ~vteClientSocket();
  QString buffer;
  int view;
  QString GetName() {return name;}
  QTcpSocket *getSocket() {return socket;}

  signals:
  void newData(QString, int);
  void logText(const QString&);

  private slots:
  void emitNewDataSignal();

private:
  int socket_id;
  QTcpSocket *socket;
  QString name;
  vteGUIWindow *parent_window;
};

/*vteServer*/
class vteServer: public QTcpServer
{
  Q_OBJECT
  public:
  vteServer(long port, const QString &name, vteGUIWindow *parent);
  ~vteServer();
 
  private:
  vteClientSocket *client_socket[MAX_CLIENTS];
  int client_socket_id[MAX_CLIENTS];
  
  int number_of_clients;
  vteGUIWindow *parent_window;

  signals:
  void newConnect(vteClientSocket*);

  private slots:
  void AcceptConnection();
  void sendBack(QString, int);
};

/*vteNetwork---------------------------*/

class vteNetwork:public QObject
{
public:
 vteNetwork(vteGUIWindow *parent, long port, const QString &name, int speed,
                                          bool server_flag, const QString &address=NULL);
 ~vteNetwork();

 int Disconnect();
 int PropagateMessage(const QString &message, int recipient=-1);


 vteServer *GetServer() {return server;}
 vteClient *GetClient() {return client;}
 bool AmIServer() {return is_server;}

private:
 vteServer *server;
 vteClient *client;
 vteGUIWindow *parent_window;

 void CheckView();

 bool connected;
 bool is_server;
};

#endif
