#ifndef _vteVisualizationWindow_h
#define _vteVisualizationWindow_h
#include "vteObject.h"
#include "vteCollection.h"
#include "vteIsosurface.h"
#include "vteCuttingPlane.h"
#include "vteFileReader.h"
#include "vteTitle.h"
#include "vteGeometry.h"
#include "vteGlyphs.h"
#include "vteGrid.h"
#include "vteScalarBar.h"
#include "vteCuttingPlane.h"
#include "vteIsosurface.h"

#include "vtkRenderer.h"
#include "vtkCoordinate.h"
#include "vtkRenderWindow.h"
#include "vtkRenderWindowInteractor.h"
#include "vtkJPEGWriter.h"
#include "vtkWindowToImageFilter.h"
#include "vtkIVExporter.h"
#include "vtkVRMLExporter.h"

#include "QVTKWidget.h"

class vteVisualizationWindow:public vteObject
{
public:
	static vteVisualizationWindow *New(QWidget *parent, const char *file_name);
	void Delete();

	int Execute();

	vtkRenderWindow *GetWindow() {return qvtk_window;}
  
	int ToggleGeometry();
	int ToggleGrid();
	int ToggleGlyphs();
	int ToggleScalarGradient();
	int TogglePlanes();
	int ToggleIsosurface();
	int ToggleTitle();
	int ToggleScalarBar();

	void Repaint();			/*redraw after actor toggle*/


	bool GetGeometryVisibility() {return show_geometry;}
	bool GetGridVisibility() {return show_grid;}
	bool GetGlyphsVisibility() {return show_glyphs;}
	bool GetScalarGradientVisibility() {return show_scalar_gradient;}
	bool GetPlanesVisibility() {return show_planes;}
	bool GetIsosurfaceVisibility() {return show_isosurface;}
	bool GetTitleVisibility() {return show_title;}
	bool GetScalarBarVisibility() {return show_scalar_bar;}

	vteIsosurfaceForVtk* GetIsosurfaceForVtk() {return isosurface_for_vtk;}
	vteCuttingPlaneCollection* GetCuttingPlaneCollection() {return cutting_plane_collection;}
	vteTitleForVtk *GetTitleForVtk() {return title_for_vtk;}
	vteScalarBarForVtk *GetScalarBarForVtk() {return scalar_bar_for_vtk;}
	void SetScalarBarForVtk(vteScalarBarForVtk *scalar_bar) {scalar_bar_for_vtk = scalar_bar;}

	void Render() {renderer->Render();}
	vtkRenderer *GetRenderer() {return renderer;}
	QVTKWidget *GetWidget() {return qvtk_widget;}

	bool HasGeometry() {return IsDefined(input->GetGeometryCollection());}
	bool HasGlyphs() {return IsDefined(input->GetGlyphSetCollection());}
	bool HasGrid() {return IsDefined(input->GetGridFrameCollection());}
	bool HasIsosurface() {return isosurface_for_vtk;}
	bool HasCuttingPlanes() {return cutting_plane_collection;}

	bool HasFrames() {return true;}//return particles->input->frame->GetMaxFrame()>1?true:false;}

	int WriteJPEG(char *file_name,int quality=90);
	int WriteIV(char *file_name);
	int WriteVRML(char *file_name);

	void UpdateToFrame(long frame_index);
	int ExtrapolateScalarGradient(vteGeometryForVtk *geometry_for_vtk);

	long GetCurrentFrame() {return current_frame;}
	long GetMaxFrame() {return max_frame;}

	vteFileReader *GetInput() {return input;}

protected:
	vteVisualizationWindow();
	~vteVisualizationWindow();

private:
	bool running;

	bool show_glyphs;
	bool show_grid;
	bool show_geometry;
	bool show_title;
	bool show_scalar_gradient;
	bool show_isosurface;
	bool show_planes;
	bool show_scalar_bar;

	unsigned int grid_component_index;

	int current_frame;
	int max_frame;
	int max_grid_frame;
	int *max_glyph_frame;	/*FIX THIS*/

	vteFileReader *input; /*should make this more generic by defining base class*/

	QWidget *parent;
	QVTKWidget *qvtk_widget;
	vtkRenderWindow *qvtk_window;

	vtkRenderer *renderer;
	//vtkCoordinate *syscoordinate;

	/*VTK data structures*/
	vteIsosurfaceForVtk *isosurface_for_vtk;
	vteCuttingPlaneCollection *cutting_plane_collection;
	vteTitleForVtk *title_for_vtk;
	vteScalarBarForVtk *scalar_bar_for_vtk;
};

#endif
