#include "vteCuttingPlane.h"
#include "vtkProperty.h"
#include "vtkPolyData.h"

vteCuttingPlane::vteCuttingPlane()
{
	contour_lut=NULL;
	plane=NULL;
	cutter=NULL;
	cutter_mapper=NULL;
	contour_mapper=NULL;
	cutter_actor=NULL;
	contour_actor=NULL;
	grid_for_vtk=NULL;
}

vteCuttingPlane::~vteCuttingPlane()
{
	if (contour_lut!=NULL) contour_lut->Delete();
	if (plane!=NULL) plane->Delete();
	if (cutter!=NULL) cutter->Delete();
	if (cutter_mapper!=NULL) cutter_mapper->Delete();
	if (contour_mapper!=NULL) contour_mapper->Delete();
	if (cutter_actor!=NULL) cutter_actor->Delete();
	if (contour_actor!=NULL) contour_actor->Delete();
}

vteCuttingPlane *vteCuttingPlane::New(vteGridForVtk *grid_for_vtk)
{
	vteCuttingPlane *cutting_plane=new vteCuttingPlane();
	if (cutting_plane==NULL) return NULL;
 
	/*make LUT for contour lines*/
	cutting_plane->contour_lut=vtkLookupTable::New();
	cutting_plane->contour_lut->SetNumberOfColors(1);
	cutting_plane->contour_lut->SetValueRange(0,0);
	cutting_plane->contour_lut->Build();

	cutting_plane->plane=vtkPlane::New();
	if (cutting_plane->plane==NULL) return NULL;

	double *vtk_pts;
	vtePoint origin;
	vteVector normal;

	vtk_pts=grid_for_vtk->GetStructuredGrid()->GetCenter();
	VEC_COPY(origin.x,vtk_pts);
	VEC_ZERO(normal.u);
	normal.u[0]=1;
	
	cutting_plane->plane->SetOrigin(grid_for_vtk->GetStructuredGrid()->GetCenter());
	cutting_plane->plane->SetNormal(normal.u);
	cutting_plane->SetOrigin(&origin);
	cutting_plane->SetNormal(&normal);

	cutting_plane->cutter=vtkCutter::New();
	if (cutting_plane->cutter==NULL) return NULL;
	cutting_plane->cutter->SetInput(grid_for_vtk->GetStructuredGrid());
	cutting_plane->cutter->SetCutFunction(cutting_plane->plane);

	cutting_plane->filter=vtkContourFilter::New();
	if (cutting_plane->filter==NULL) return NULL;
	cutting_plane->filter->SetInput(cutting_plane->cutter->GetOutput());
	cutting_plane->filter->ComputeScalarsOff();
	cutting_plane->filter->GenerateValues(10,grid_for_vtk->GetUserRange()->min,grid_for_vtk->GetUserRange()->max);

	cutting_plane->cutter_mapper=vtkPolyDataMapper::New();
	if (cutting_plane->cutter_mapper==NULL) return NULL;

	cutting_plane->cutter_mapper->SetInput(cutting_plane->cutter->GetOutput());
	cutting_plane->cutter_mapper->SetScalarRange(grid_for_vtk->GetUserRange()->min,grid_for_vtk->GetUserRange()->max);
	cutting_plane->cutter_mapper->SetLookupTable(grid_for_vtk->GetLut());

	cutting_plane->contour_mapper=vtkPolyDataMapper::New();
	if (cutting_plane->contour_mapper==NULL) return NULL;
	cutting_plane->contour_mapper->SetInput(cutting_plane->filter->GetOutput());
	cutting_plane->contour_mapper->SetScalarRange(grid_for_vtk->GetUserRange()->min,grid_for_vtk->GetUserRange()->max);
	cutting_plane->contour_mapper->SetLookupTable(cutting_plane->contour_lut);

	cutting_plane->cutter_actor=vtkLODActor::New();
	if (cutting_plane->cutter_actor==NULL) return NULL;
	cutting_plane->cutter_actor->SetMapper(cutting_plane->cutter_mapper);
	cutting_plane->cutter_actor->GetProperty()->SetOpacity(1);

	cutting_plane->contour_actor=vtkLODActor::New();
	if (cutting_plane->contour_actor==NULL) return NULL;
	cutting_plane->contour_actor->SetMapper(cutting_plane->contour_mapper);
	cutting_plane->contour_actor->GetProperty()->SetOpacity(1);
	cutting_plane->contour_actor->GetProperty()->SetColor(0,0,0);

	cutting_plane->grid_for_vtk=grid_for_vtk;
	Define(cutting_plane);
	return cutting_plane;
}

void vteCuttingPlane::Modified()
{
	UpdatePlane();
}

void vteCuttingPlane::Delete()
{
 delete this;
}

void vteCuttingPlane::SetOrigin(vtePoint *o)
{
	VEC_COPY(origin.x,o->x);
}

void vteCuttingPlane::SetNormal(vteVector *n)
{
	VEC_COPY(normal.u,n->u);
}

//---collection
vteCuttingPlaneCollection *vteCuttingPlaneCollection::New()
{
  vteCuttingPlaneCollection *cutting_plane_collection=new vteCuttingPlaneCollection;
  if (cutting_plane_collection==NULL) return NULL;
  Define(cutting_plane_collection);
  return cutting_plane_collection;
}

void vteCuttingPlaneCollection::Delete()
{
  delete this;
}

vteCuttingPlaneCollection::vteCuttingPlaneCollection()
{
	cutting_plane=NULL;
}

vteCuttingPlaneCollection::~vteCuttingPlaneCollection()
{
	for (int i=0;i<members;i++)
		delete cutting_plane[i];

	delete cutting_plane;
}

vteCuttingPlane *vteCuttingPlaneCollection::NewCuttingPlane(vteGridForVtk *grid_for_vtk)
{
	if (!cutting_plane)
		cutting_plane = new (vteCuttingPlane*);
	else
		cutting_plane = (vteCuttingPlane**) realloc(cutting_plane,sizeof(vteCuttingPlane**)*(members+1));

	cutting_plane[members]=vteCuttingPlane::New(grid_for_vtk);
	members++;
	return cutting_plane[members-1];
}

void vteCuttingPlaneCollection::RemoveCuttingPlane(int i)
{
  if ((i>=0) && (i<members))
  {
    cutting_plane[i]->Delete();
    int p;
    for(p=i;p<members-1;p++) cutting_plane[i]=cutting_plane[i+1];
    members--;
  } 
}    
  
vteCuttingPlane *vteCuttingPlaneCollection::GetCuttingPlane(int i)
{
  if ((i>=0) && (i<members)) return cutting_plane[i];
return NULL;
}


void vteCuttingPlane::UpdatePlane()
{
	
	filter->GenerateValues(10,grid_for_vtk->GetUserRange()->min,grid_for_vtk->GetUserRange()->max);
	cutter_mapper->SetScalarRange(grid_for_vtk->GetUserRange()->min,grid_for_vtk->GetUserRange()->max);
	contour_mapper->SetScalarRange(grid_for_vtk->GetUserRange()->min,grid_for_vtk->GetUserRange()->max);
	//cutter_mapper->SetLookupTable(grid_for_vtk->GetLut());
	//contour_mapper->SetLookupTable(cutting_plane->contour_lut);
	cutter_mapper->Modified();
	contour_mapper->Modified();


}
