#include "vteDialogGridProperties.h"
#include <iostream>

#include <QTextStream>
#include <QLabel>
#include <QTextStream>
#include <QGroupBox>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QGridLayout>
#include <QStandardItemModel>
using namespace std;

vteDialogGridProperties::vteDialogGridProperties(unsigned int noc, unsigned int active, vteRange *absolute_r,
    vteRange *user_r,QString *names, QWidget* parent, Qt::WFlags fl): QDialog( parent, fl )
{

	setCaption( tr( "Grid Properties" ) );

	QVBoxLayout *main = new QVBoxLayout();

	/*active component*/
	QGroupBox *ac_group = new QGroupBox (tr("Active Component"));
    QVBoxLayout *vbox1 = new QVBoxLayout();

	listComponent = new QListView();
	QStringList entries;

	QStandardItemModel *model = new QStandardItemModel();
 	for (unsigned int i=0;i<noc;i++)
		model->appendRow(new QStandardItem(names[i]));

	listComponent->setModel(model);
	
	QModelIndex index = model->index(active,0);
	listComponent->selectionModel()->select(index,QItemSelectionModel::Select);

	vbox1->addWidget(listComponent);
	ac_group->setLayout(vbox1);
	main->addWidget(ac_group);
    
	/*range*/
	QGroupBox *range_group = new QGroupBox(tr("Range"));
	QGridLayout *grid = new QGridLayout();

	QLabel *min_label = new QLabel(tr("Min:"));
	QLabel *max_label = new QLabel(tr("Max:"));
    sliderMin = new QSlider( );
    sliderMin->setOrientation( Qt::Horizontal );
    sliderMax = new QSlider( );
    sliderMax->setOrientation( Qt::Horizontal );
	textMin = new QLineEdit();
	textMin->setMinimumWidth(70);
	textMin->setMaximumWidth(70);
	
	textMax = new QLineEdit();
	textMax->setMinimumWidth(70);
	textMax->setMaximumWidth(70);
	
	grid->addWidget(min_label,0,0);
	grid->addWidget(sliderMin,0,1);
	grid->addWidget(textMin,0,2);
	grid->addWidget(max_label,1,0);
	grid->addWidget(sliderMax,1,1);
	grid->addWidget(textMax,1,2);

	range_group->setLayout(grid);
	main->addWidget(range_group);

    /*resolution*/
	QGroupBox *res_box = new QGroupBox(tr("Resolution"));
	QVBoxLayout *vbox2 = new QVBoxLayout();
	    
    radioDefault = new QRadioButton(tr("Default"));
    radioDefault->setChecked( TRUE );

    radio8 = new QRadioButton( tr("1/8th size"));
    radio8->setEnabled( FALSE );
    
    radio64 = new QRadioButton( tr("1/64th size") );
    radio64->setEnabled( FALSE );
 
	vbox2->addWidget(radioDefault);
	vbox2->addWidget(radio8);
	vbox2->addWidget(radio64);
	res_box->setLayout(vbox2);
	main->addWidget(res_box);
    
	/*buttons*/
	QWidget *buttons = new QWidget();
	QHBoxLayout *hbox1 = new QHBoxLayout;

    buttonOK = new QPushButton(tr("&OK"));
    buttonOK->setDefault( TRUE );
	buttonCancel = new QPushButton(tr("Cancel"));
	hbox1->addStretch();
	hbox1->addWidget(buttonOK);
	hbox1->addWidget(buttonCancel);
	hbox1->addStretch();
	buttons->setLayout(hbox1);
	main->addWidget(buttons);
    
    // tab order
    setTabOrder( sliderMin, textMin );
    setTabOrder( textMin, sliderMax );
    setTabOrder( sliderMax, textMax );
    setTabOrder( textMax, radioDefault );
    setTabOrder( radioDefault, buttonOK );
    setTabOrder( buttonOK, buttonCancel );

	active_component=active;
	absolute_range=absolute_r;
    user_range=user_r;

    sliderMin->setMinValue(0);
		sliderMin->setMaxValue(100);

  	sliderMax->setMinValue(0);
		sliderMax->setMaxValue(100);

    InitValues();

    updating=false;
  // signals and slots connections
    connect( buttonOK, SIGNAL( clicked() ), this, SLOT( accept() ) );
    connect( buttonCancel, SIGNAL( clicked() ), this, SLOT( reject() ) );
    connect( sliderMin, SIGNAL( valueChanged(int) ), this, SLOT( SliderMinMoved() ) );
    connect( sliderMax, SIGNAL( valueChanged(int) ), this, SLOT( SliderMaxMoved() ) );
    connect( textMax, SIGNAL( textChanged(const QString&) ), this, SLOT( UpdateSliders() ) );
    connect( textMin, SIGNAL( textChanged(const QString&) ), this, SLOT( UpdateSliders() ) );
	connect( listComponent->selectionModel(), SIGNAL( selectionChanged(const QItemSelection&, const QItemSelection&)),this,SLOT(InitValues()));
	
	setLayout(main);

}

/*
 *  Destroys the object and frees any allocated resources
 */
vteDialogGridProperties::~vteDialogGridProperties()
{
    // no need to delete child widgets, Qt does it all for us
}

/*
 *  Sets the strings of the subwidgets using the current
 *  language.
 */
void vteDialogGridProperties::languageChange()
{

}

void vteDialogGridProperties::InitValues()
{
    double u_min,u_max,a_min,a_max;

    updating=true;

	QModelIndexList rows = listComponent->selectionModel()->selectedRows();
    QModelIndex row;

	/*there should be only one*/
	if (rows.count()<1) return;

	active_component=rows.first().row();
    
	a_min=absolute_range[active_component].min;
	a_max=absolute_range[active_component].max;
    u_min=user_range[active_component].min;
	u_max=user_range[active_component].max;

	sliderMin->setValue((int)(100.0*(u_min-a_min)/(a_max-a_min)));
	sliderMax->setValue((int)(100.0*(u_max-a_min)/(a_max-a_min)));

	QString s1,s2;
	QTextStream ts1(&s1,QIODevice::WriteOnly);
	QTextStream ts2(&s2,QIODevice::WriteOnly);
	ts1<<u_min;
	s1.truncate(6);
	textMin->setText(s1);

    ts2<<u_max;
	s2.truncate(6);
	textMax->setText(s2);

	updating=false;
}

void vteDialogGridProperties::SliderMinMoved()
{
  if (updating) return;
  updating=true;

	if (sliderMin->value()>sliderMax->value()) sliderMin->setValue(sliderMax->value());
  updating=false;
	UpdateTextFields();
}

void vteDialogGridProperties::SliderMaxMoved()
{
  if (updating) return;
  updating=true;

  if (sliderMax->value()<sliderMin->value()) sliderMax->setValue(sliderMin->value());
  updating=false;
  UpdateTextFields();
}


void vteDialogGridProperties::UpdateTextFields()
{
  if (updating) return;
  updating=true;

  QString s1(""),s2("");
  int min,max;
  min=sliderMin->value();
  max=sliderMax->value();

  QTextStream ts1(&s1,QIODevice::ReadWrite);
  QTextStream ts2(&s2,QIODevice::ReadWrite);
  ts1.setRealNumberPrecision(3);
  ts2.setRealNumberPrecision(3);

  float percent;
  float range_min=absolute_range[active_component].min;
  float range_max=absolute_range[active_component].max;

  percent=(float)min/100.0;
  user_range[active_component].min=(range_max-range_min)*percent+range_min;
  ts1<<user_range[active_component].min;
  s1.truncate(6);
  textMin->setText(s1);

  percent=(float)max/100.0;
  user_range[active_component].max=(range_max-range_min)*percent+range_min;
  ts2<<user_range[active_component].max;
  s2.truncate(6);
  textMax->setText(s2);

  updating=false;
}

//------
void vteDialogGridProperties::UpdateSliders()
{
	float min, max,percent,range_min, range_max;
	QString s_min, s_max;
	if (updating) return;

	updating=true;

	s_min=textMin->text();
	s_max=textMax->text();

	QTextStream t_min(&s_min,QIODevice::ReadOnly);
	QTextStream t_max(&s_max,QIODevice::ReadOnly);
	QString corrected_min,corrected_max;
	QTextStream tc_min (&corrected_min,QIODevice::WriteOnly);
	QTextStream tc_max (&corrected_max,QIODevice::WriteOnly);

	tc_min.precision(5);tc_max.precision(5);
	t_min.precision(5);t_max.precision(5);

	range_min=absolute_range[active_component].min;
	range_max=absolute_range[active_component].max;

	t_min>>min; t_max>>max;

	if (min<range_min)
	{
		min=range_min;
		tc_min<<min;
		textMin->setText(corrected_min);
	}

	if (max>range_max)
	{
		max=range_max;
		tc_max<<max;
		textMax->setText(corrected_max);
	}

	if (min>max)
	{
		min=max;
		tc_min<<min;
		textMin->setText(corrected_min);
	}

	if (max<min)
	{
		max=min;
		tc_max<<max;
		textMax->setText(corrected_max);
	}

	user_range[active_component].min=min;
	user_range[active_component].max=max;

	percent=100.0*(min-range_min)/(range_max-range_min);
	sliderMin->setValue((int)percent);

	percent=100.0*(max-range_min)/(range_max-range_min);
	sliderMax->setValue((int)percent);

	updating=false;
}

