#include "vteDialogIsoProperties.h"

#include <QHBoxLayout>
#include <QVBoxLayout>

vteDialogIsoProperties::vteDialogIsoProperties(bool automatic,unsigned int nol, QString &levels,unsigned int transparency,
	 QWidget* parent, Qt::WFlags fl ): QDialog( parent, fl )

{
    setCaption( tr( "Isosurface Properties" ) );
    
	groupBox = new QGroupBox( tr("Contour Levels") );
	QVBoxLayout *grid = new QVBoxLayout;

    radioAuto = new QRadioButton(tr("Automatic"));
	grid->addWidget(radioAuto);

	QWidget *widget1 = new QWidget;
	QHBoxLayout *hbox1 = new QHBoxLayout;
    textLabel4 = new QLabel(tr("Number of levels:" ));
	
    spinContourLevels = new QSpinBox( );
    spinContourLevels->setMinValue(1);
    spinContourLevels->setMaxValue(300);
    spinContourLevels->setValue(nol);
	spinContourLevels->setMaximumWidth(50);
	hbox1->addWidget(textLabel4);
	//hbox1->addSpacing(10);
	hbox1->addWidget(spinContourLevels);
	hbox1->addStretch();
	widget1->setLayout(hbox1);
	grid->addWidget(widget1);
    
	radioManual = new QRadioButton( tr("Manual") );
	grid->addWidget(radioManual);

	textLabel1 = new QLabel(tr("Enter space delimited contour levels:"));
	textLevels = new QLineEdit( QString::null);
	textLevels->setMinimumWidth(350);

	grid->addWidget(textLabel1);
	grid->addWidget(textLevels);
        
	groupBox->setLayout(grid);

	QWidget *trBox = new QWidget;
	QHBoxLayout *hbox = new QHBoxLayout;
	textLabel3 = new QLabel( tr("Transparency:") );
	
	spinTransparency = new QSpinBox( );
    spinTransparency->setMaxValue( 100 );
    spinTransparency->setLineStep( 5 );
    spinTransparency->setValue(transparency);
	spinTransparency->setMaximumWidth(50);
	
	hbox->addWidget(textLabel3);
	hbox->addWidget(spinTransparency);
	hbox->addStretch();

	trBox->setLayout(hbox);
	
    QWidget *buttons = new QWidget;
	QHBoxLayout *vbox = new QHBoxLayout;
	buttonCancel = new QPushButton(tr("&Cancel"));
    buttonOK = new QPushButton(tr("&OK"));
    buttonOK->setDefault( TRUE );
	vbox->addStretch(1);
	vbox->addWidget(buttonOK);
	vbox->addWidget(buttonCancel);
	vbox->addStretch(1);
	buttons->setLayout(vbox);
    
	/*add our groups to the GUI*/
	QVBoxLayout *layout = new QVBoxLayout;
	layout->addWidget(groupBox);
	layout->addWidget(trBox);
	layout->addWidget(buttons);
	setLayout(layout);

    languageChange();
    textLevels->setText(levels);

    // signals and slots connections
    connect( buttonOK, SIGNAL( clicked() ), this, SLOT( accept() ) );
    connect( buttonCancel, SIGNAL( clicked() ), this, SLOT( reject() ) );
    connect( radioManual, SIGNAL( toggled(bool) ), textLevels, SLOT( setEnabled(bool) ) );
    connect( radioAuto, SIGNAL( toggled(bool) ), spinContourLevels, SLOT( setEnabled(bool) ) );

    if (automatic)
			 {
				radioAuto->setChecked(TRUE);
				textLevels->setEnabled(FALSE);
			 }
		else
			{
				radioManual->setChecked(TRUE);
				spinContourLevels->setEnabled(FALSE);
			}

    // tab order
    setTabOrder( radioAuto, spinContourLevels );
    setTabOrder( spinContourLevels, textLevels );
    setTabOrder( textLevels, spinTransparency );
    setTabOrder( spinTransparency, buttonOK );
    setTabOrder( buttonOK, buttonCancel );
}

/*
 *  Destroys the object and frees any allocated resources
 */
vteDialogIsoProperties::~vteDialogIsoProperties()
{
    // no need to delete child widgets, Qt does it all for us
}

/*
 *  Sets the strings of the subwidgets using the current
 *  language.
 */
void vteDialogIsoProperties::languageChange()
{
}
