#include "vteDialogPlanesProperties.h"
#include "vteCollection.h"

#include <strstream>
#include <iostream>

#include <QGroupBox>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QGridLayout>
#include <QPushButton>
#include <QTextStream>

using namespace std;

vteDialogPlanesProperties::vteDialogPlanesProperties( vtePlaneDefinition *plane_def, int nop,
					QWidget* parent, Qt::WFlags fl )
    : QDialog( parent, fl )

{
	QVBoxLayout *main = new QVBoxLayout;
    setCaption( tr( "Plane Properties" ) );

	QGroupBox *group_box = new QGroupBox(tr("Cutting Planes"));
	QHBoxLayout *group_layout = new QHBoxLayout();

	/* make the "left" side, containing planes list and add/delete buttons*/
	QWidget *planes_widget = new QWidget();
	QGridLayout *planes_layout = new QGridLayout();

	listPlanes = new QListView( );
	planes_layout->addWidget(listPlanes,0,0,1,2);

    QPushButton *buttonAdd = new QPushButton( tr ("&Add"));
    QPushButton *buttonDelete = new QPushButton( tr("&Delete") );
    planes_layout->addWidget(buttonAdd,1,0);
	planes_layout->addWidget(buttonDelete,1,1);
	planes_widget->setLayout(planes_layout);

	/* make the "right" side, containing plane properties*/
	QWidget *props_widget = new QWidget();
	QVBoxLayout *props_layout = new QVBoxLayout();
	
	/*make the top part, the vector data*/
	QWidget *vector_widget = new QWidget();
	QGridLayout *vector_layout = new QGridLayout();
	
    textX = new QLineEdit();
    textY = new QLineEdit();
    textZ = new QLineEdit();
    textI = new QLineEdit();
    textJ = new QLineEdit();
    textK = new QLineEdit();

	textX->setMaximumWidth(50);
	textY->setMaximumWidth(50);
	textZ->setMaximumWidth(50);
	textI->setMaximumWidth(50);
	textJ->setMaximumWidth(50);
	textK->setMaximumWidth(50);
	
	QLabel *labelX = new QLabel("x:");
	QLabel *labelY = new QLabel("y:");
	QLabel *labelZ = new QLabel("z:");
	QLabel *labelI = new QLabel("i:");
	QLabel *labelJ = new QLabel("j:");
	QLabel *labelK = new QLabel("k:");
	
	vector_layout->addWidget(labelX,0,0);
	vector_layout->addWidget(textX,0,1);
	vector_layout->addWidget(labelI,0,2);
	vector_layout->addWidget(textI,0,3);

	vector_layout->addWidget(labelY,1,0);
	vector_layout->addWidget(textY,1,1);
	vector_layout->addWidget(labelJ,1,2);
	vector_layout->addWidget(textJ,1,3);

	vector_layout->addWidget(labelZ,2,0);
	vector_layout->addWidget(textZ,2,1);
	vector_layout->addWidget(labelK,2,2);
	vector_layout->addWidget(textK,2,3);
	vector_widget->setLayout(vector_layout);
	props_layout->addWidget(vector_widget);

	/*make the bottom part with the spinners*/
	QWidget *levels_widget = new QWidget();
	QGridLayout *levels_layout = new QGridLayout();

	QLabel *labelLevels = new QLabel("Number of levels:");
	labelLevels->setAlignment(Qt::AlignRight);
	QLabel *labelTransp = new QLabel("Transparency:");
	labelTransp->setAlignment(Qt::AlignRight);
	
	spinContourLevels = new QSpinBox();
	spinContourLevels->setMaximumWidth(50);
    spinTransparency = new QSpinBox();
    spinTransparency->setMaxValue( 100 );
    spinTransparency->setLineStep( 5 );
	spinTransparency->setMaximumWidth(50);

	levels_layout->addWidget(labelLevels,0,0);
	levels_layout->addWidget(spinContourLevels,0,1);
	levels_layout->addWidget(labelTransp,1,0);
	levels_layout->addWidget(spinTransparency,1,1);

	levels_widget->setLayout(levels_layout);
	props_layout->addWidget(levels_widget);
	props_widget->setLayout(props_layout);
	
	/*add the left and right widget*/
	group_layout->addWidget(planes_widget);
	group_layout->addWidget(props_widget);
	group_box->setLayout(group_layout);

	/*OK and Cancel Buttons*/
	QWidget *buttons = new QWidget();
	QHBoxLayout *buttons_layout = new QHBoxLayout();
	
    buttonOK = new QPushButton(tr("&OK"));
    buttonOK->setDefault( TRUE );

    buttonCancel = new QPushButton(tr("&Cancel") );
	buttons_layout->addStretch();
	buttons_layout->addWidget(buttonOK);
	buttons_layout->addWidget(buttonCancel);
	buttons->setLayout(buttons_layout);

	/*finish the layout*/
	main->addWidget(group_box);
	main->addWidget(buttons);
	setLayout(main);
	
    // tab order
    setTabOrder( listPlanes, textX );
    setTabOrder( textX, textY );
    setTabOrder( textY, textZ );
    setTabOrder( textZ, textI );
    setTabOrder( textI, textJ );
    setTabOrder( textJ, textK );
    setTabOrder( textK, buttonAdd );
    setTabOrder( buttonAdd, buttonDelete );
    setTabOrder( buttonDelete, buttonOK );
    setTabOrder( buttonOK, buttonCancel );

    /*populate planes list*/
	plane_definition=plane_def;

	QString string;
	QTextStream str(&string);

	model = new QStandardItemModel();
 	for (int i=0;i<nop;i++)
	{
		str<<"Plane "<<i+1;	
		model->appendRow(new QStandardItem(string));
	}
	listPlanes->setModel(model);
	
	/*select first plane*/
	QModelIndex index = model->index(0,0);
	listPlanes->selectionModel()->select(index,QItemSelectionModel::Select);

    //listPlanes->setCurrentItem(0);
    UpdateTextFields();

    // signals and slots connections
    connect( buttonAdd, SIGNAL( clicked() ), this, SLOT( AddPlane() ) );
    connect( buttonDelete, SIGNAL( clicked() ), this, SLOT( DeletePlane() ) );
    connect( buttonOK, SIGNAL( clicked() ), this, SLOT( accept() ) );
    connect( buttonCancel, SIGNAL( clicked() ), this, SLOT( reject() ) );
    connect( listPlanes->selectionModel(), SIGNAL( selectionChanged(const QItemSelection&, const QItemSelection&) ), this, SLOT (UpdateTextFields()));
    connect( textX, SIGNAL(textChanged(const QString&)), this, SLOT(UpdatePlaneDefinitions()));
    connect( textY, SIGNAL(textChanged(const QString&)), this, SLOT(UpdatePlaneDefinitions()));
    connect( textZ, SIGNAL(textChanged(const QString&)), this, SLOT(UpdatePlaneDefinitions()));
    connect( textI, SIGNAL(textChanged(const QString&)), this, SLOT(UpdatePlaneDefinitions()));
    connect( textJ, SIGNAL(textChanged(const QString&)), this, SLOT(UpdatePlaneDefinitions()));
    connect( textK, SIGNAL(textChanged(const QString&)), this, SLOT(UpdatePlaneDefinitions()));
    connect( spinContourLevels, SIGNAL(valueChanged(int)), this, SLOT(UpdatePlaneDefinitions()));
    connect( spinTransparency, SIGNAL(valueChanged(int)), this, SLOT(UpdatePlaneDefinitions()));

}

/*
 *  Destroys the object and frees any allocated resources
 */
vteDialogPlanesProperties::~vteDialogPlanesProperties()
{
    // no need to delete child widgets, Qt does it all for us
}

/*
 *  Sets the strings of the subwidgets using the current
 *  language.
 */
void vteDialogPlanesProperties::languageChange()
{
}

void vteDialogPlanesProperties::UpdateTextFields()
{
	int i=0;		/*index of plane to change*/
	int transparency;
	updating=true;

	QString string;
	QTextStream str(&string);

	if (listPlanes->model()->rowCount()>0)
		i = listPlanes->selectionModel()->selectedRows().first().row();

	str.setRealNumberPrecision(3);
	str<<plane_definition[i].origin.x[0];
	textX->setText(string);

	string.clear();
	str<<plane_definition[i].origin.x[1];
	textY->setText(string);

	string.clear();
	str<<plane_definition[i].origin.x[2];
	textZ->setText(string);
	
	string.clear();
	str<<plane_definition[i].normal.u[0];
	textI->setText(string);

	string.clear();
	str<<plane_definition[i].normal.u[1];
	textJ->setText(string);

	string.clear();
	str<<plane_definition[i].normal.u[2];
	textK->setText(string);

	spinContourLevels->setValue(plane_definition[i].number_of_levels);
	transparency=(int)((1.0-plane_definition[i].alpha)*100);
	spinTransparency->setValue(transparency);
	updating=false;
}

void vteDialogPlanesProperties::UpdatePlaneDefinitions()
{
	if (!updating)
	{
		int i=listPlanes->selectionModel()->selectedRows().first().row();
  
		plane_definition[i].origin.x[0]=textX->text().toFloat();
		plane_definition[i].origin.x[1]=textY->text().toFloat();
		plane_definition[i].origin.x[2]=textZ->text().toFloat();
		plane_definition[i].normal.u[0]=textI->text().toFloat();
		plane_definition[i].normal.u[1]=textJ->text().toFloat();
		plane_definition[i].normal.u[2]=textK->text().toFloat();
		plane_definition[i].number_of_levels=spinContourLevels->value();
		plane_definition[i].alpha=(100.0f-(float)spinTransparency->value())/100.0f;
	}
}

void vteDialogPlanesProperties::AddPlane()
{
	QString string;
	QTextStream name(&string);
	int nop = listPlanes->model()->rowCount();
	
	name<<"Plane " <<nop+1<<'\0';

	model->appendRow(new QStandardItem(string));
	
	/*select*/
	QModelIndex index = model->index(nop,0);
	
	VEC_ZERO(plane_definition[nop].origin.x);
    plane_definition[nop].normal.u[0]=1;
    plane_definition[nop].normal.u[1]=0;
    plane_definition[nop].normal.u[2]=0;
    plane_definition[nop].number_of_levels=10;
    plane_definition[nop].alpha=1;
    
	/*need to do this here, since this will triger update using above values*/
	listPlanes->selectionModel()->select(index,QItemSelectionModel::ClearAndSelect);
	nop++;
}

void vteDialogPlanesProperties::DeletePlane()
{
	int nop;
	int current,i;

	nop=listPlanes->model()->rowCount();

    //*****

    if (nop==1) return;

    current=listPlanes->selectionModel()->selectedRows().first().row();
	model->removeRow(current);

    for(i=current;i<nop-1;i++)
	{
		VEC_COPY(plane_definition[i].origin.x,plane_definition[i+1].origin.x);
		VEC_COPY(plane_definition[i].normal.u,plane_definition[i+1].normal.u);
		plane_definition[i].alpha=plane_definition[i+1].alpha;
		plane_definition[i].number_of_levels=plane_definition[i+1].number_of_levels;
	}
	if (current>0) current--;
	QModelIndex index=model->index(current,0);
	listPlanes->selectionModel()->setCurrentIndex(index,QItemSelectionModel::ClearAndSelect);
}
