#include "vteFileReader.h"

#include <iostream>
#include <fstream>
#include <string>
using namespace std;

#include <memory.h>
#include <float.h>
#include <stdlib.h>

//constructors / destructor
vteFileReader::vteFileReader()
{
	input_file_name[0]=0;
	title=NULL;
	description=NULL;
}

vteFileReader::~vteFileReader()
{
	if (title!=NULL) delete title;
	if (description!=NULL) delete description;
}

vteFileReader* vteFileReader::New(const char *file_name)
{
	vteFileReader *file_reader=new vteFileReader;
	strcpy(file_reader->input_file_name,file_name);
	return file_reader;
}

void vteFileReader::Delete()
{
	delete this;
}

//Execute - overhead for data reading
int vteFileReader::Execute()
{
	//check if already executed
	if (this->HasExecuted()==true)
		return WarningMessage("FileReader already executed");
	
	if (input_file_name==NULL)
		return ErrorMessage("input file name not set");
	
	if (Read(input_file_name)==vteERROR)
		return vteERROR;
	
	if (description!=NULL) Message("Processed data set for ",description);

	executed=true;
	return vteOK;
}

//Read - reads data from file
int vteFileReader::Read(char *file_name)
{
	char dir[1000];
	char name[1000];
	char *end_of_directory;

	Message("Processing file ",file_name);

	end_of_directory=strrchr(file_name,'\\'); //find last '\'
	if (!end_of_directory)
		end_of_directory=strrchr(file_name,'/'); //try the unix one

	if (end_of_directory!=NULL)
	{
		int characters;

		/*split into dir and name*/
		characters=(int)(end_of_directory-file_name+1);
		memcpy(dir,file_name,characters);
		dir[characters]=0;		//terminate string;

		strcpy(name,end_of_directory+1);
	}
	else		//file_name does not contain directory reference
	{
		dir[0]=0;		//empty string;
		strcpy(name,file_name);
	}	

	/*open file for reading*/
	ifstream file(file_name);

	if (file==NULL)
		return ErrorMessage("cannot open file ",name," in directory ",dir);

	//process file
	char command[50];
	char text[255];
	char ascii[50];
	double version;
	
	//read in header
	file>>ws;
	file>>command;

	if (stricmp(command,"VTE")==0)
	{
		file>>version;
		file>>ascii;

		/*little hack, since I went back with version numbers*/
		version=VERSION;

		if (version>VERSION)	 //unsupported version
		{
			sprintf(text,"%3.2f",version);
			return ErrorMessage("unsupported data version ",text);
		}

		if (stricmp(ascii,"ASCII")!=0)
			return ErrorMessage(" unrecognized file type ",ascii,". Only ASCII is supported by this version");
	}

	/*process file*/
	file>>ws>>command;
	while(file)
	{
		if (command[0]=='#')		//comment, ignore this line
			file.ignore(10000,'\n');
		else if (stricmp(command,"INSERT")==0)
		{
			file>>text;
			sprintf(file_name,"%s%s",dir,text);
			if (Read(file_name) == vteERROR)		//call Read recursively
				return vteERROR;
		}
		else if (stricmp(command,"TITLE")==0)
		{
			if (title==NULL) title=new char[51];
			if (title==NULL) return ErrorMessage("insufficient memory");
			file>>ws;
			file.getline(title,50,'\n');
		}
		else if (stricmp(command,"DESCRIPTION")==0)
		{
			if (description==NULL) description=new char[500];
			if (description==NULL) return ErrorMessage("insufficient memory");
			file>>ws;
			file.getline(description,500,'\n');
		}
		else if (stricmp(command, "GLYPHS")==0)
		{
			if (ReadGlyphSet(file)==vteERROR)
				return vteERROR;
		}
		else if (stricmp(command, "GRID")==0)
		{
			if (ReadGrid(file)==vteERROR)
				return vteERROR;
		}
		else if (stricmp(command, "GEOMETRY")==0)
		{
			if (ReadGeometry(file)==vteERROR)
				return vteERROR;
		}
		else
		{
			warning=true;
			WarningMessage("unrecognized command: ",command);
			return vteWARNING;
		}
	file>>ws>>command;
	}

	file.close();

	if (warning==false) return vteOK;
	else return vteWARNING;
}

//ReadGlyphSet - process GLYPHS block
int vteFileReader::ReadGlyphSet(ifstream &file)
{
	#define PVT 1		//x y z u v w t
	#define PT 2		//x y z t

	char command[50];
	char source[50];
	int type;
	int i, nog;

	bool proceed=true;
	vteGlyphFrameCollection *frame_collection;

	/*add new item to the glyph set collection*/
	frame_collection = glyph_set_collection.AddNew();

	/*figure out what sort of data we got*/
	file>>command;
	if (stricmp(command,"pvt")==0)
		type=PVT;
	else if (stricmp(command,"pt")==0)
		type=PT;
	else
		return ErrorMessage("unrecognized glyph type ",command);

	/*start processing the file*/
	file>>command;
	proceed=true;
	while(proceed)
	{
		/* definition of glyph shapes*/
		if (stricmp(command,"GLYPHS_GEOMETRY")==0)
		{
    		int number_of_types;
			vteColor color;
			double size;
			
			file>>number_of_types;
			
    		for (i=0;i<number_of_types;i++)
    		{
  				file>>source;
				file>>size;
				file>>color.r>>color.g>>color.b>>color.a;
				
  				if (stricmp(source,"cloud")==0) frame_collection->AddShape(CLOUD,size,color);
				else if (stricmp(source,"triangle")==0) frame_collection->AddShape(TRIANGLE,size,color);
				else if (stricmp(source,"wedge")==0) frame_collection->AddShape(WEDGE,size,color);
  				else if (stricmp(source,"lo_res_cone")==0) frame_collection->AddShape(LO_RES_CONE,size,color);
  				else if (stricmp(source,"med_res_cone")==0) frame_collection->AddShape(MED_RES_CONE,size,color);
  				else if (stricmp(source,"hi_res_cone")==0) frame_collection->AddShape(HI_RES_CONE,size,color);
  				else if (stricmp(source,"cube")==0) frame_collection->AddShape(CUBE,size,color);
	  			else if (stricmp(source,"lo_res_sphere")==0) frame_collection->AddShape(LO_RES_SPHERE,size,color);
	  			else if (stricmp(source,"med_res_sphere")==0) frame_collection->AddShape(MED_RES_SPHERE,size,color);
				else if (stricmp(source,"hi_res_sphere")==0) frame_collection->AddShape(HI_RES_SPHERE,size,color);
				else return ErrorMessage("unrecognized glyph geometry",source);
			}

			file>>ws;
			file>>command;

			if (stricmp(command,"GLYPHS_GEOMETRY_END")!=0)
  	  			WarningMessage("expected GLYPHS_GEOMETRY_END, found ",command);
		}    //end of GLYPH_GEOMETRY
		
		/*glyph data*/
		else if (stricmp(command,"FRAME")==0)
		{
			file>>nog;
			
			/*allocate new frame*/
			vteGlyphFrame *frame = frame_collection->AddFrame(nog);
			vteGlyph glyph;

			switch (type)
			{
			case PVT: for(i=0;i<nog;i++)
					  {
						  file>>glyph.x[0];
						  file>>glyph.x[1];
						  file>>glyph.x[2];
						  file>>glyph.u[0];
						  file>>glyph.u[1];
						  file>>glyph.u[2];
						  file>>glyph.t;
						  glyph.t--;

						  /* add the glyph to the frame*/
						  frame->AddGlyph(&glyph);
					  }	break;
			case PT: for(i=0;i<nog;i++)
					 {
						 file>>glyph.x[0];
						 file>>glyph.x[1];
						 file>>glyph.x[2];
						 file>>glyph.t;
						 glyph.t--;
						 VEC_ZERO(glyph.u);
						 glyph.u[2]=1;

						 /* add the glyph to the frame*/
						 frame->AddGlyph(&glyph);
					 } break;
			}

			file>>ws;
			file>>command;
			if (stricmp(command,"FRAME_END")!=0)
				WarningMessage("expected FRAME_END, instead found ",command);
		}
		else if (stricmp(command,"GLYPHS_END")==0)
		{
			proceed=false;
		}
		else
		    WarningMessage("unknown command ",command);
	
		file>>ws;
		file>>command;
	}

	frame_collection->SetCurrentFrame(0);
	if (!frame_collection->GetNumberOfShapes()) 
		return ErrorMessage("glyph set did not define geometry");

	Define(&glyph_set_collection);
	Define(frame_collection);
	return vteOK;
}

//ReadGeometry - processes GEOMETRY block
int vteFileReader::ReadGeometry(ifstream &file)
{
	char type[20];
	char command[50];
	bool proceed;
	int i;
	vtePoint offset;

	VEC_ZERO(offset.x);
	file>>type;

	vteGeometry *geometry=geometry_collection.NewGeometry();
	if (geometry==NULL) return ErrorMessage("insufficient memory for vteGeometry");
 	
	if (stricmp(type,"triangular_mesh")==0)
	{
		file>>ws;
		file>>command;
		proceed=true;

		while(proceed)
		{
			if (stricmp(command,"GEOMETRY_COLORS")==0)
			{
				int number_of_colors;
				vteColor color;

				file>>number_of_colors;
	
      			for (i=0;i<number_of_colors;i++)
      			{
    				file>>color.r;
    				file>>color.g;
    				file>>color.b;
					file>>color.a;
					geometry->AddColor(color);
				}

      			file>>ws;
      			file>>command;

      			if (stricmp(command,"GEOMETRY_COLORS_END")!=0)
		      		WarningMessage("expected GEOMETRY_COLORS_END, found ",command);
			}
			else if (stricmp(command,"OFFSET")==0)
			{
				file>>offset.x[0];
				file>>offset.x[1];
				file>>offset.x[2];
			}
			else if (stricmp(command,"NODES")==0)
			{
				int number_of_nodes;
				file>>number_of_nodes;

				vteNode *node=new vteNode[number_of_nodes];
				if (!node)
					return ErrorMessage("insufficient memory for geometry nodes");
				
				file>>ws;

				//read in nodes
				for (i=0;i<number_of_nodes;i++)
				{
					file>>node[i].x[0];
					file>>node[i].x[1];
					file>>node[i].x[2];
					file>>node[i].color_index;

					VEC_ADD(node[i].x,offset.x);
	
					node[i].color_index--;		//start with 0;
				}
				geometry->AddNodes(node,number_of_nodes);
				delete node;

				file>>ws;
				file>>command;

				if (stricmp(command,"NODES_END")!=0)
					WarningMessage("expected NODES_END, instead found ",command);
			}
			else if (stricmp(command,"CELLS")==0)
			{
				int number_of_cells;
				file>>number_of_cells;

				vteCell *cell = new vteCell[number_of_cells];
				if (!cell)
					return ErrorMessage("ReadGeometry - insufficient memory for cells");
				
				file>>ws;
		        //read in cells
				for (i=0;i<number_of_cells;i++)
				{
					file>>cell[i].node1;
					file>>cell[i].node2;
					file>>cell[i].node3;

					cell[i].node1--;
					cell[i].node2--;
					cell[i].node3--;
				}

				geometry->AddCells(cell, number_of_cells);
			
				file>>ws;
				file>>command;

				if (stricmp(command,"CELLS_END")!=0)
					WarningMessage("expected CELLS_END, instead found ",command);
			}
			else if (stricmp(command,"GEOMETRY_END")==0)
			{
				proceed=false;
			}
			else 
			{	
				WarningMessage("unrecognized command: ",command);
 				if (!file) return ErrorMessage("corrupted GEOMETRY block suspected");
			}

			file>>ws;
			file>>command;
		}	//end of while(proceed)
	}	//end of triangular_mesh
	else
		return ErrorMessage("unrecognized geometry type ",type);

		if (!geometry->GetNumberOfNodes())
			return ErrorMessage("geometry block did not define nodes");

		if (!geometry->GetNumberOfCells())
			return ErrorMessage("geometry block did not define cells");

	Define(geometry);
	return vteOK;
}


//--------------ReadGrid-------------
int vteFileReader::ReadGrid(ifstream &file)
{
	char command[50];
	char name[25];
	bool proceed;
	unsigned int Nx,Ny,Nz;
	double x,y,z;

	unsigned n_vars;
	unsigned long i,j,nodes=0,frames;
	
	vteRange *absolute_range=grid_frame_collection.GetAbsoluteRange();
	vteRange *user_range=grid_frame_collection.GetUserRange();

	proceed=true;
	frames=0;

	while(proceed)
	{
		file>>ws;
		file>>command;

  		if (stricmp(command,"NODES")==0)
		{
			if (IsDefined(&grid_frame_collection))
				return ErrorMessage("attempt to redefine grid topography");
			
			file>>Nx>>Ny>>Nz;
			nodes=Nx*Ny*Nz;
			grid_frame_collection.SetNumberOfNodes(Nx,Ny,Nz);
		}
		else if (stricmp(command,"SPACING")==0)
		{
			file>>x>>y>>z;
			grid_frame_collection.SetSpacing(x,y,z);
		}

		else if (stricmp(command,"ORIGIN")==0)
		{
			file>>x>>y>>z;
			grid_frame_collection.SetOrigin(x,y,z);
		}
		else if (stricmp(command,"COMPONENTS")==0)
		{
			file>>n_vars;
			file>>ws;
			if (n_vars>50) return ErrorMessage("this version supports maximum 50 variables");
			for (i=0;i<n_vars;i++)
			{
				file>>name;
				if (grid_frame_collection.AddVar(name)==vteERROR)
					return vteERROR;
			}
	
			for (i=0;i<n_vars;i++)
			{
				absolute_range[i].min=FLT_MAX;
				absolute_range[i].max=-FLT_MAX;
			}
		}
		else if (stricmp(command,"FRAME")==0)
		{
			vteGridData *data,val;
			vteGridFrame *grid_frame = grid_frame_collection.AddFrame();
			
			data = new vteGridData[n_vars*nodes];

			for (i=0;i<nodes;i++)
				for (j=0;j<n_vars;j++)
				{
					file>>val;
					
					data[nodes*j+i]=val;
					
					if (val<absolute_range[j].min) absolute_range[j].min=val;
					if (val>absolute_range[j].max) absolute_range[j].max=val;
				}

			if (grid_frame->SetVarBlock(data))
				return vteERROR;

			delete data;

			file>>ws;
			file>>command;

			if (stricmp(command,"FRAME_END")!=0)
			{
				WarningMessage("expected FRAME_END, found ",command);
				warning=true;
			}

		}
		else if (stricmp(command,"GRID_END")==0)
		{
			proceed=false;
		}
		else
		{
			return ErrorMessage("unknown command ",command);
			if (!file) return ErrorMessage("corrupted GRID block suspected");
		}

	}

	for (i=0;i<n_vars;i++)           //start with maximum range
	 {
		user_range[i].min=absolute_range[i].min;
		user_range[i].max=absolute_range[i].max;
	 }

	Define(&grid_frame_collection);
	return vteOK;
}

/*GetChecksum1 & GetChecksum2

the following produce some numbers that are used to identify the data set
this is used to verify that all clients are looking at the same data
*/
unsigned long vteFileReader::GetCheckSum1()
{
	unsigned long checksum=0;

	checksum+=geometry_collection.GetNumberOfMembers();
	checksum+=glyph_set_collection.GetNumberOfMembers();
	checksum+=grid_frame_collection.GetNumberOfVars();
 
	return checksum;
}

unsigned long vteFileReader::GetCheckSum2()
{
	unsigned long checksum=0;
	int i;

	for (i=0;i<geometry_collection.GetNumberOfMembers();i++)
		checksum+=geometry_collection.GetGeometry(i)->GetNumberOfNodes();
    
	for (i=0;i<glyph_set_collection.GetNumberOfMembers();i++)
		checksum+=glyph_set_collection.Get(i)->GetFrame(0)->GetNumberOfMembers();
    
	checksum+=grid_frame_collection.GetNx()*grid_frame_collection.GetNy()*grid_frame_collection.GetNz();

	return checksum;
}
