//***************************************
//***************************************
qtVisualizationWindow::qtVisualizationWindow( QWidget* parent, QStatusBar *bar, const char* file_name)
    : QMainWindow( parent )
{
	window=vteVisualizationWindow::New(this,file_name);
	if (window==NULL) close();
	window->SetStatusBar(bar);
	status_bar=bar;
	if (window->Execute()==vteERROR) close();

	resize(300,300);
	show();
   
	//widget->show();
	setFocusProxy(window->GetWidget());
	setCentralWidget(window->GetWidget());

	playback_rate=3;
	playback_timer = new QTimer(this);
	connect( playback_timer, SIGNAL(timeout()), this, SLOT(PlaybackTimerOverhead()) );
	playing=false;

	record_name_index=0;
	recording=false;
	record_timer=new QTimer(this);
	connect(record_timer, SIGNAL(timeout()), this, SLOT(RecordTimerOverhead()));

	network=NULL;
	network_status = new vteNetworkStatus(this);
	network_connected=false;

	iso_properties_auto=true;
//   parent_gui=gui;
}

qtVisualizationWindow::~qtVisualizationWindow()
{
	if (playback_timer!=NULL) {delete playback_timer;playback_timer=NULL;}
	if (window!=NULL) window->Delete();
	if (network!=NULL) {delete network;network=NULL;}

	if (record_timer!=NULL) {delete record_timer; record_timer=NULL;}
	//if (view_timer!=NULL) {delete view_timer;view_timer=NULL;}
	if (status_bar!=NULL) {delete status_bar;status_bar=NULL;}
	if (network_status!=NULL) {delete network_status;network_status=NULL;}
}

//PlaybackTimerOverhead - updates to next frame
void qtVisualizationWindow::PlaybackTimerOverhead()
{
	window->UpdateToFrame(window->GetCurrentFrame()+1);
}

int qtVisualizationWindow::TogglePlay()
{
	if (playing)
	{
		playback_timer->stop();
			playing=false;
	}
	else
	{
	  playback_timer->start((int)(1000.0/playback_rate),FALSE);
	  playing=true;
	}
	
	return vteOK;
}

void qtVisualizationWindow::SetPlaybackRate(int rate)
{
	playback_rate=rate;
	if (playing) playback_timer->changeInterval((int)(1000.0/playback_rate));
}

//starts recording snapshots at a preset interval
int qtVisualizationWindow::ToggleRecord()
{
	if (recording)
	{
		record_timer->stop();
		recording=false;
	}                        
	else
	{
		record_timer->start(250,FALSE);
		recording=true;
	}
	
	return vteOK;
}

void qtVisualizationWindow::RecordTimerOverhead()
{
	strstream name;
	name<<"capVTE_recording_";
	if (record_name_index<1000)
	{
		name<<"0";
		if (record_name_index<100)
		{
			name<<"0";
			if (record_name_index<10)	name<<"0";
		}
	}

	name<<record_name_index<<".jpg"<<'\0';
  window->WriteJPEG(name.str(),100);    //no compression
  record_name_index++;
}

int qtVisualizationWindow::StartNetwork(long port, const QString &name,
             int speed, bool server, const QString &address)
{
  network = new vteNetwork(this, port,name,speed,server,address);

  return vteOK;
}

void qtVisualizationWindow::StopNetwork()
{
 delete network;
 network_connected=false;
}

void qtVisualizationWindow::NetworkConnectionStatus(int type)
{
  switch (type)
  {
    case ACCEPT: network_connected=true;
                 view_timer=new QTimer(network->GetClient());
                 status_bar->message("connection accepted");
                 GetNetworkStatus()->textview->append(tr("Client started successfully.\n"));
                 connect(network->GetClient(),SIGNAL(NewData(QString &)),this,SLOT(ProcessNetworkData(QString &)));
                 //if (network->AmIServer())
                 //{
                   connect(view_timer,SIGNAL(timeout()),SLOT(PropagateView()));
                   view_timer->start(network->GetClient()->GetSpeed(),FALSE);
                 //}
                 break;
    case DATA_MISMATCH: status_bar->message("data mismatch");
                 network_connected=true;
                 break;
    case REJECT: network_connected=false;
                 cout<<"Mismatched network protocol versions, please update!"<<endl;
                 cout<<"Check http://www.aoe.vt.edu/groups/caplab/hpv.html for the most recent version"<<endl;
                 status_bar->message("connection refused");
                 break;
  }
}                                                                     

void qtVisualizationWindow::ProcessNetworkData(QString &data)
{
	char command[300],buf[300];
	int i,nol,rate,frame,nop;
	double x,y,z,nx,ny,nz;
	double vx,vy,vz,near,far;
	double *levels;
	long active_component;
	QColor c;
	vteRange *absolute_range,*user_range;
	QTextIStream line(&data);
	qtVisualizationWindow *active_window;
	vteGridFrame *frame2;
	vteGridForVtk *grid_for_vtk;
	vteIsosurfaceForVtk *isosurface;
	vteScalarBarForVtk *scalar_bar;

	line>>command; //this will contain "VTE"
 
	line>>command;

	active_window=(qtVisualizationWindow*)parent_gui->GetActiveWindow();
	vteVisualizationWindow *win=active_window->GetVisualizationWindow();
	vteIsosurfaceForVtk *isosurface_for_vtk=win->GetInput()->GetIsosurfaceForVtk();
	parent_gui->SetActiveWindow((vteGUIWindow*)this);
	isosurface=win->GetInput()->GetIsosurfaceForVtk();
	scalar_bar=win->GetInput()->GetScalarBarForVtk();
	grid_for_vtk=win->GetInput()->GetGridForVtk();
	vteCuttingPlaneForVtkCollection *cutting_plane_for_vtk_collection=win->GetInput()->GetCuttingPlaneForVtkCollection();
	vteCuttingPlaneForVtk *cutting_plane_for_vtk;

	network_connected=false;  //let's lie to the toggle action
  
	if (stricmp(command,"TOGGLE_GEOMETRY")==0)
			parent_gui->ToggleGeometry();
	else if (stricmp(command,"TOGGLE_GLYPHS")==0)
			parent_gui->ToggleGlyphs();
	else if (stricmp(command,"TOGGLE_GRID")==0)
			parent_gui->ToggleGrid();
	else if (stricmp(command,"TOGGLE_ISOSURFACE")==0)
          parent_gui->ToggleIsosurface();
	else if (stricmp(command,"TOGGLE_PLANES")==0)
          parent_gui->TogglePlanes();
	else if (stricmp(command,"TOGGLE_PLAY")==0)
          parent_gui->TogglePlay();
	else if (stricmp(command,"TOGGLE_SCALAR_GRADIENT")==0)
          parent_gui->ToggleScalarGradient();
	else if (stricmp(command,"VIEW")==0)
	{
		line>>command;
		x=atof(command);
		line>>command;
		y=atof(command);
		line>>command;
		z=atof(command);
		line>>command;
		nx=atof(command);
		line>>command;
		ny=atof(command);
		line>>command;
		nz=atof(command);
		line>>command;
		vx=atof(command);
		line>>command;
		vy=atof(command);
		line>>command;
		vz=atof(command);
		line>>command;
		near=atof(command);
		line>>command;
		far=atof(command);

		active_window->GetVisualizationWindow()->GetRenderer()->GetActiveCamera()->SetPosition(x,y,z);
		active_window->GetVisualizationWindow()->GetRenderer()->GetActiveCamera()->SetFocalPoint(nx,ny,nz);
		active_window->GetVisualizationWindow()->GetRenderer()->GetActiveCamera()->SetViewUp(vx,vy,vz);
		active_window->GetVisualizationWindow()->GetRenderer()->GetActiveCamera()->SetClippingRange(near,far);
   /*TODO: Fix the lights*/
		//active_window->GetVisualizationWindow()->GetRenderer()->GetLights()->SetPosition(x,y,z);
		//active_window->GetVisualizationWindow()->GetRenderer()->GetLights()->SetFocalPoint(nx,ny,nz);
		active_window->GetVisualizationWindow()->GetRenderer()->Render();
	}
	else if (stricmp(command,"ISO")==0)
	{
		line>>command;
		if (IsDefined(isosurface_for_vtk)==false) parent_gui->ToggleIsosurface();

		if (stricmp(command,"AUTO")==0)
		{
			active_window->SetIsoPropertiesAuto(true);
			line>>command;
			isosurface_for_vtk->GenerateLevels((int)atof(command));
			line>>command;
			isosurface_for_vtk->SetAlpha(atof(command));
		}
		
		line>>command;
		
		if (stricmp(command,"MANUAL")==0)
		{
			line>>command;
			nol=atoi(command);
			levels=new double[nol];
     
			//if (levels==NULL) {ErrorMessage("insufficient memory for levels"); return;}
     
			for (i=0;i<nol;i++)
			{
				line>>command;
				levels[i]=atof(command);
			}

			if (nol>0) isosurface_for_vtk->SetContours(nol,levels);
			delete levels;
			line>>command;
			isosurface_for_vtk->SetAlpha(atof(command));
		}
		
		active_window->repaint(TRUE);
	}
	else if (stricmp(command,"PLAYBACK")==0)
	{
		line>>command;
		rate=atoi(command);
		line>>command;
		frame=atoi(command);
		active_window->SetPlaybackRate(rate);
		win->UpdateToFrame(frame-1);
		active_window->repaint(TRUE);
	}
	else if (stricmp(command,"GRID")==0)
	{
		frame2=grid_for_vtk->GetInput();
		absolute_range=grid_for_vtk->GetAbsoluteRange(0);
		user_range=grid_for_vtk->GetUserRange(0);
		line>>command;
		active_component=atol(command);
		frame2->SetActiveComponent(active_component);
		vteGeometryForVtkCollection *geometry_for_vtk_collection=win->GetInput()->GetGeometryForVtkCollection();
		
		bool gradient_visible=win->GetScalarGradientVisibility();

		active_window->GetVisualizationWindow()->GetRenderer()->RemoveActor(scalar_bar->GetActor());
		active_window->GetVisualizationWindow()->GetRenderer()->RemoveActor(grid_for_vtk->GetActor());
		active_window->GetVisualizationWindow()->GetRenderer()->RemoveActor(isosurface->GetActor());
		//active_window->GetVisualizationWindow()->GetRenderer()->RemoveActor(cutting_plane_for_vtk_collection->GetCuttingPlaneForVtk(i)->GetContourActor());
		//active_window->GetVisualizationWindow()->GetRenderer()->RemoveActor(cutting_plane_for_vtk_collection->GetCuttingPlaneForVtk(i)->GetCutterActor());
		
		if(gradient_visible==true)
		{
			for (i=0;i<geometry_for_vtk_collection->GetNumberOfMembers();i++)
				active_window->GetVisualizationWindow()->GetRenderer()->RemoveActor(geometry_for_vtk_collection->GetGeometryForVtk(i)->GetActor());
		}
  
		win->GetInput()->Execute();
		grid_for_vtk=win->GetInput()->GetGridForVtk();
		isosurface=win->GetInput()->GetIsosurfaceForVtk();
		scalar_bar=win->GetInput()->GetScalarBarForVtk();
		cutting_plane_for_vtk_collection=win->GetInput()->GetCuttingPlaneForVtkCollection();
		frame2=grid_for_vtk->GetInput();
  
		//active_window->GetVisualizationWindow()->GetRenderer()->AddActor(grid_for_vtk->GetActor());
		//active_window->GetVisualizationWindow()->GetRenderer()->AddActor(isosurface->GetActor());
		//active_window->GetVisualizationWindow()->GetRenderer()->AddActor(cutting_plane_for_vtk_collection->GetCuttingPlaneForVtk(i)->GetContourActor());
		//active_window->GetVisualizationWindow()->GetRenderer()->AddActor(cutting_plane_for_vtk_collection->GetCuttingPlaneForVtk(i)->GetCutterActor());
		
		active_window->GetVisualizationWindow()->GetRenderer()->AddActor(scalar_bar->GetActor());

		if(gradient_visible==true)
		{

			if (gradient_visible) win->ToggleScalarGradient();
			vteGeometryForVtkCollection *geometry_for_vtk_collection=win->GetInput()->GetGeometryForVtkCollection();
			vteGeometryForVtk *geometry_for_vtk;
			for (i=0;i<geometry_for_vtk_collection->GetNumberOfMembers();i++)
			{
				geometry_for_vtk=geometry_for_vtk_collection->GetGeometryForVtk(i);
				win->ExtrapolateScalarGradient(geometry_for_vtk);
				geometry_for_vtk->GetMapper()->Modified();
			}
			
			if (gradient_visible) win->ToggleScalarGradient();
		}

		vteCuttingPlaneForVtkCollection *cutting_plane_for_vtk_collection=win->GetInput()->GetCuttingPlaneForVtkCollection();
		vteCuttingPlaneForVtk *cutting_plane_for_vtk;

		bool cutting_plane_visible=win->GetPlanesVisibility();

		if (cutting_plane_visible==true) win->TogglePlanes();

		for (i=0;i<cutting_plane_for_vtk_collection->GetNumberOfMembers();i++)
		{
			cutting_plane_for_vtk=cutting_plane_for_vtk_collection->GetCuttingPlaneForVtk(i);
			cutting_plane_for_vtk->GetFilter()->GenerateValues(cutting_plane_for_vtk->GetFilter()->GetNumberOfContours(),user_range[i].min,user_range[i].max);
			cutting_plane_for_vtk->GetCutter()->Modified();
			cutting_plane_for_vtk->GetPlane()->Modified();
		}

		if (cutting_plane_visible==true) win->TogglePlanes();
		//scalar_bar->GetActor()->SetTitle(frame2->GetComponentName(active_component));

		isosurface->GetFilter()->GenerateValues(5,user_range[active_component].min, user_range[active_component].max);
		isosurface->GetMapper()->SetScalarRange(user_range[active_component].min,user_range[active_component].max);
		isosurface->GetFilter()->Modified();

		grid_for_vtk->GetLut()->SetTableRange(user_range[active_component].min, user_range[active_component].max);
		grid_for_vtk->GetLut()->Modified();

	active_window->repaint(TRUE);
	}
	else if (stricmp(command,"COMMAND")==0)
	{
		line>>command;
		strcpy(buf,"> ");
		strcat(buf,command);
		line>>command;
		strcat(buf," ");
		strcat(buf,command);
		c=QColor(0,0,200);
		active_window->GetNetworkStatus()->textview->setColor(c);
		active_window->GetNetworkStatus()->textview->append(buf);
		c=QColor(0,0,0);
		active_window->GetNetworkStatus()->textview->setColor(c);
	}
	else if (stricmp(command,"DISCONNECTED")==0 && (!network->AmIServer()))
	{
		line>>command;
		strcpy(buf,command);
		strcat(buf," has left the server.");
		c=QColor(200,0,0);
		active_window->GetNetworkStatus()->textview->setColor(c);
		active_window->GetNetworkStatus()->textview->append(buf);
		c=QColor(0,0,0);
		active_window->GetNetworkStatus()->textview->setColor(c);
	}
	else if (stricmp(command,"CONNECTED")==0 && (!network->AmIServer()))
	{
		line>>command;
		strcpy(buf,command);
		strcat(buf," is joining the server.");
		c=QColor(200,0,0);
		active_window->GetNetworkStatus()->textview->setColor(c);
		active_window->GetNetworkStatus()->textview->append(buf);
		c=QColor(0,0,0);
		active_window->GetNetworkStatus()->textview->setColor(c);
	}
	else if (stricmp(command,"PLANES")==0)
	{
		int nop_dialog;
		float alpha;
		bool planes_visible=win->GetPlanesVisibility();
		
		line>>command;
		nop_dialog=atoi(command);
		vtePoint point;
		vteVector vector;
		vteRange *range=win->GetInput()->GetGridForVtk()->GetUserRange();
		nop=cutting_plane_for_vtk_collection->GetNumberOfMembers();

		if (planes_visible) win->TogglePlanes();

		if (nop_dialog>nop)
			for (i=nop;i<nop_dialog;i++) cutting_plane_for_vtk_collection->NewCuttingPlaneForVtk(grid_for_vtk);
		else if (nop_dialog<nop)
			for (i=nop_dialog;i<nop;i++) cutting_plane_for_vtk_collection->RemoveCuttingPlaneForVtk(i);

		for(i=0;i<nop_dialog;i++)
		{
			line>>command;
			point.x=atof(command);
			line>>command;
			point.y=atof(command);
			line>>command;
			point.z=atof(command);

			line>>command;
			vector.i=atof(command);
			line>>command;
			vector.j=atof(command);
			line>>command;
			vector.k=atof(command);

			line>>command;
			nol=atoi(command);
			line>>command;
			alpha=atof(command);

			cutting_plane_for_vtk=cutting_plane_for_vtk_collection->GetCuttingPlaneForVtk(i);
			cutting_plane_for_vtk->GetPlane()->SetOrigin(point.x, point.y, point.z);
			cutting_plane_for_vtk->GetPlane()->SetNormal(vector.i,vector.j,vector.k);

			cutting_plane_for_vtk->GetFilter()->GenerateValues(nol,range->min,range->max);
			cutting_plane_for_vtk->GetCutterActor()->GetProperty()->SetOpacity(alpha);
			cutting_plane_for_vtk->GetContourActor()->GetProperty()->SetOpacity(alpha);
			cutting_plane_for_vtk->SetOrigin(&point);
			cutting_plane_for_vtk->SetNormal(&vector);
			cutting_plane_for_vtk->GetCutter()->Modified();
			cutting_plane_for_vtk->GetPlane()->Modified();
		}
    
		if (planes_visible) win->TogglePlanes();
		active_window->repaint(TRUE);
	}

	network_connected=true;
	parent_gui->SetActiveWindow((vteGUIWindow*)active_window);
}

void qtVisualizationWindow::PropagateView()
{
 double x,y,z,nx,ny;
 double nz,vx,vy,vz;
 double near,far;
 char X[7],Y[7],Z[7],NX[7],NY[7],NZ[7];
 char VX[7],VY[7],VZ[7],NEAR[7],FAR[7];
 char message[100];
 QString view;
 qtVisualizationWindow *active_window;

 active_window=(qtVisualizationWindow*)parent_gui->GetActiveWindow();
 parent_gui->SetActiveWindow((vteGUIWindow*)this);

   active_window->GetVisualizationWindow()->GetRenderer()->GetActiveCamera()->GetPosition(x,y,z);
   active_window->GetVisualizationWindow()->GetRenderer()->GetActiveCamera()->GetFocalPoint(nx,ny,nz);
   active_window->GetVisualizationWindow()->GetRenderer()->GetActiveCamera()->GetViewUp(vx,vy,vz);
   active_window->GetVisualizationWindow()->GetRenderer()->GetActiveCamera()->GetClippingRange(near,far);

   gcvt(x,6,X);
   gcvt(y,6,Y);
   gcvt(z,6,Z);
   gcvt(nx,6,NX);
   gcvt(ny,6,NY);
   gcvt(nz,6,NZ);
   gcvt(vx,6,VX);
   gcvt(vy,6,VY);
   gcvt(vz,6,VZ);
   gcvt(near,6,NEAR);
   gcvt(far,6,FAR);

   strcpy(message,"VTE VIEW ");

   strcat(message,X);
   strcat(message," ");
   strcat(message,Y);
   strcat(message," ");
   strcat(message,Z);
   strcat(message," ");
   strcat(message,NX);
   strcat(message," ");
   strcat(message,NY);
   strcat(message," ");
   strcat(message,NZ);
   strcat(message," ");
   strcat(message,VX);
   strcat(message," ");
   strcat(message,VY);
   strcat(message," ");
   strcat(message,VZ);
   strcat(message," ");
   strcat(message,NEAR);
   strcat(message," ");
   strcat(message,FAR);

   view=QString(message);
   network->PropagateMessage(view,-1);
   //}

}
