/*
 capVTE GUI
*/
#include "filesave.xpm"
#include "fileopen.xpm"
#include "icoGeometry.xpm"
#include "icoGrid.xpm"
#include "icoGlyphs.xpm"
#include "icoIsosurface.xpm"
#include "icoPlane.xpm"
#include "icoScalarGradient.xpm"
#include "icoPlay.xpm"
#include "icocapVTE.xpm"
#include "icoRecord.xpm"
#include "icoNetwork.xpm"
#include "icoRedBlue.xpm"
#include "icoCrystalEyes.xpm"
#include "icoNormalView.xpm"

#include <strstream>
#include "vteGUI.h"
#include "vteDialogGlyphProperties.h"
#include "vteDialogPlanesProperties.h"
#include "vteDialogGridProperties.h"
#include "vteDialogIsoProperties.h"
#include "vteDialogPlaybackProperties.h"

#include "vteDialogNetworkStart.h"
#include "vteNetwork.h"

#include "vteGUIWindow.h"
#include "vteHelpWindow.h"

#include <QTimer>
#include <QMessageBox>
#include <QComboBox>
#include <QMenuItem>
#include <QFileDialog>
#include <QTextStream>
#include <QDir>

#include <vtkProperty.h>
#include <vtkCamera.h>
#include <vtkLightCollection.h>

using namespace std;

#define gcvt _gcvt

vteGUI::vteGUI()
{
	toggle_action_disabled=false;

	statusBar();
	setName( "vteGUI" );
	setIcon( QPixmap( icocapVTE ) );

	// actions
	actionOpen = new QAction( this, "actionOpen" );
	actionOpen->setIconSet( QIcon( QPixmap( fileopen ) ) );
	actionSave = new QAction( this, "actionSave" );
	actionSave->setIconSet( QIcon( QPixmap(filesave ) ) );
    actionClose = new QAction( this, "actionClose" );
    actionExit = new QAction( this, "actionExit" );
    
	actionToggleGeometry = new QAction( this, "actionToggleGeometry" );
    actionToggleGeometry->setToggleAction( TRUE );
    actionToggleGeometry->setIconSet( QIcon( QPixmap(icoGeometry ) ) );
    actionToggleGlyphs = new QAction( this, "actionToggleGlyphs" );
    actionToggleGlyphs->setToggleAction( TRUE );
    actionToggleGlyphs->setIconSet( QIcon( QPixmap( icoGlyphs) ) );
    actionToggleGrid = new QAction( this, "actionToggleGrid" );
    actionToggleGrid->setToggleAction( TRUE );
    actionToggleGrid->setOn( FALSE );
    actionToggleGrid->setIconSet( QIcon( QPixmap(icoGrid) ) );
    actionTogglePlanes = new QAction( this, "actionTogglePlanes" );
    actionTogglePlanes->setToggleAction( TRUE );
    actionTogglePlanes->setIconSet( QIcon( QPixmap(icoPlane) ) );
    actionToggleIsosurface = new QAction( this, "actionToggleIsosurface" );
    actionToggleIsosurface->setToggleAction( TRUE );
    actionToggleIsosurface->setIconSet( QIcon( QPixmap(icoIsosurface) ) );
    actionIsoProperties = new QAction( this, "actionIsoProperties" );
	actionGlyphProperties = new QAction( this,"actionGlyphProperties");
    actionGridProperties = new QAction( this, "actionGridProperties" );
    actionPlanesProperties = new QAction( this, "actionPlanesProperties" );
    actionPlaybackProperties = new QAction( this, "actionPlaybackProperties" );
    
	actionTogglePlay = new QAction( this, "actionTogglePlay" );
    actionTogglePlay->setToggleAction( TRUE );
    actionTogglePlay->setIconSet( QIcon( QPixmap( icoPlay) ) );
    actionToggleRecord=new QAction(this,"actionToggleRecord");
    actionToggleRecord->setToggleAction(TRUE);
    actionToggleRecord->setIconSet(QIcon( QPixmap( icoRecord)));
    
	actionHelp = new QAction( this, "actionHelp" );
    actionAbout = new QAction( this, "actionAbout" );
    
	actionToggleScalarGradient = new QAction( this, "actionToggleScalarGradient" );
    actionToggleScalarGradient->setToggleAction( TRUE );
    actionToggleScalarGradient->setIconSet( QIcon( QPixmap(icoScalarGradient) ) );

    actionToggleNetwork = new QAction(this, "actionToggleNetwork");
    actionToggleNetwork->setToggleAction(TRUE);
    actionToggleNetwork->setIconSet(QIcon(QPixmap(icoNetwork)));
	
    actionToggleRedBlue = new QAction(this, "actionToggleRedBlue");
    actionToggleRedBlue->setToggleAction(TRUE);
    actionToggleRedBlue->setIconSet(QIcon(QPixmap(icoRedBlue)));
    actionToggleCrystalEyes = new QAction(this, "actionToggleCrystalEyes");
    actionToggleCrystalEyes->setToggleAction(TRUE);
    actionToggleCrystalEyes->setIconSet(QIcon(QPixmap(icoCrystalEyes)));
    actionToggleNormalView = new QAction(this, "actionToggleNormalView");
    actionToggleNormalView->setToggleAction(TRUE);
    actionToggleNormalView->setIconSet(QIcon(QPixmap(icoNormalView)));

    // toolbars
    toolbarFile = addToolBar("File");
	actionOpen->addTo( toolbarFile );
    actionSave->addTo( toolbarFile );
	
    toolbarToggle = addToolBar("Toggle");
    actionToggleGeometry->addTo( toolbarToggle );
    actionToggleGlyphs->addTo( toolbarToggle );
    actionToggleGrid->addTo( toolbarToggle );
    actionToggleScalarGradient->addTo( toolbarToggle );
    actionTogglePlanes->addTo( toolbarToggle );
    actionToggleIsosurface->addTo( toolbarToggle );

    toolbarView = addToolBar("View");
    actionToggleRedBlue->addTo( toolbarView );
    actionToggleCrystalEyes->addTo( toolbarView );
    actionToggleNormalView->addTo( toolbarView );

    toolbarPlay = addToolBar("Play");
    actionTogglePlay->addTo( toolbarPlay );
    actionToggleRecord->addTo(toolbarPlay);

    toolbarNetwork = addToolBar("Network");
    actionToggleNetwork->addTo( toolbarNetwork );

    // menubar
    menubar = menuBar();

    fileMenu = menubar->addMenu("File");
    actionOpen->addTo( fileMenu );
    actionSave->addTo( fileMenu );
    fileMenu->insertSeparator();
    actionClose->addTo( fileMenu );
    fileMenu->insertSeparator();
    actionExit->addTo( fileMenu );

	menubar->insertSeparator();
    
    ViewMenu = menubar->addMenu("&View");
    actionToggleGeometry->addTo( ViewMenu );
    actionToggleGlyphs->addTo( ViewMenu );
    actionToggleGrid->addTo( ViewMenu );
    actionToggleScalarGradient->addTo( ViewMenu );
    actionTogglePlanes->addTo( ViewMenu );
    actionToggleIsosurface->addTo( ViewMenu );
    
    SettingsMenu = menubar->addMenu( "&Settings" );
    actionGlyphProperties->addTo(SettingsMenu );
	actionGridProperties->addTo( SettingsMenu );
    actionPlanesProperties->addTo( SettingsMenu );
    actionIsoProperties->addTo( SettingsMenu );

    PlaybackMenu = menubar->addMenu("&Playback");
    actionTogglePlay->addTo( PlaybackMenu );
    actionToggleRecord->addTo(PlaybackMenu);
    PlaybackMenu->insertSeparator();
    actionPlaybackProperties->addTo( PlaybackMenu );
	
	NetworkMenu = menubar->addMenu("&Network");
    actionToggleNetwork->addTo(NetworkMenu);
	
    HelpMenu = menubar->addMenu("&Help");
    actionHelp->addTo( HelpMenu );
    HelpMenu->insertSeparator();
	actionAbout->addTo(HelpMenu);
    
    languageChange();
    resize( QSize(600, 480).expandedTo(minimumSizeHint()) );

    // signals and slots connections
    connect( actionToggleGeometry, SIGNAL( toggled(bool) ), this, SLOT( ToggleGeometry() ) );
    connect( actionToggleGrid, SIGNAL( toggled(bool) ), this, SLOT( ToggleGrid() ) );
    connect( actionToggleIsosurface, SIGNAL( toggled(bool) ), this, SLOT( ToggleIsosurface() ) );
    connect( actionToggleGlyphs, SIGNAL( toggled(bool) ), this, SLOT( ToggleGlyphs() ) );
    connect( actionTogglePlanes, SIGNAL( toggled(bool) ), this, SLOT( TogglePlanes() ) );
    connect( actionToggleScalarGradient, SIGNAL( toggled(bool) ), this, SLOT( ToggleScalarGradient() ) );
    connect( actionTogglePlay, SIGNAL( toggled(bool) ), this, SLOT(TogglePlay()));
    connect( actionToggleRecord, SIGNAL (toggled(bool)), this, SLOT(ToggleRecord()));
    connect( actionToggleRedBlue, SIGNAL (toggled(bool)), this, SLOT(ToggleRedBlue()));
    connect( actionToggleCrystalEyes, SIGNAL (toggled(bool)), this, SLOT(ToggleCrystalEyes()));
    connect( actionToggleNormalView, SIGNAL (toggled(bool)), this, SLOT(ToggleNormalView()));

    connect( actionToggleNetwork, SIGNAL( toggled(bool) ), this, SLOT( ToggleNetwork() ) );

    connect( actionOpen, SIGNAL( activated() ), this, SLOT( OpenDataset() ) );
    connect( actionSave, SIGNAL( activated() ), this, SLOT( SaveDataset() ) );
    connect( actionClose, SIGNAL( activated() ), this, SLOT( Close() ) );
    connect( actionGlyphProperties, SIGNAL (activated() ), this, SLOT( GetGlyphProperties() ) );
    connect( actionGridProperties, SIGNAL( activated() ), this, SLOT( GetGridProperties() ) );
    connect( actionIsoProperties, SIGNAL( activated() ), this, SLOT( GetIsoProperties() ) );
    connect( actionPlanesProperties, SIGNAL( activated() ), this, SLOT( GetPlaneProperties() ) );
    connect( actionPlaybackProperties, SIGNAL( activated() ), this, SLOT( GetPlaybackProperties() ) );
    connect( actionHelp, SIGNAL( activated() ), this, SLOT( ShowHelp() ) );
    connect( actionAbout, SIGNAL( activated() ), this, SLOT( About() ) );
	connect( actionExit, SIGNAL( activated() ), this, SLOT(close()));

	//setup workspace
    workspace = new QMdiArea();
	workspace->setHorizontalScrollBarPolicy(Qt::ScrollBarAsNeeded);
    workspace->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);
	setCentralWidget(workspace);

    DisableToolbar();
	statusBar()->message( "Ready", 2000 );

	active_window=NULL;			
}

/*
 *  Destroys the object and frees any allocated 
resources
 */
vteGUI::~vteGUI()
{
    // no need to delete child widgets, Qt does it all for us
}

/*
 *  Sets the strings of the subwidgets using the current
 *  language.
 */
void vteGUI::languageChange()
{
    setCaption( tr( "capVTE (BETA)" ) );
    actionOpen->setText( tr( "Open" ) );
    actionOpen->setMenuText( tr( "&Open..." ) );
    actionOpen->setStatusTip( tr( "Opens a new data set" ) );
    actionOpen->setAccel( tr( "Ctrl+O" ) );
    actionSave->setText( tr( "Save" ) );
    actionSave->setMenuText( tr( "&Save..." ) );
    actionSave->setToolTip( tr( "Save" ) );
    actionSave->setStatusTip( tr( "Saves the current view as a .jpg, .vrml or .iv file" ) );
    actionSave->setAccel( tr( "Ctrl+S" ) );
    actionClose->setText( tr( "Close" ) );
    actionClose->setMenuText( tr( "&Close" ) );
    actionClose->setStatusTip( tr( "Closes the current window" ) );
    actionExit->setText( tr( "Exit" ) );
    actionExit->setMenuText( tr( "E&xit" ) );
    actionExit->setToolTip( tr( "Exit" ) );
    actionExit->setStatusTip( tr( "Quits the program" ) );
	actionExit->setAccel( tr(" Ctrl+Q") );
    actionToggleGeometry->setText( tr( "Geometry" ) );
    actionToggleGeometry->setMenuText( tr( "&Geometry") );
    actionToggleGeometry->setStatusTip( tr( "Toggles the geometry visibility" ) );
    actionToggleGlyphs->setText( tr( "Glyphs" ) );
    actionToggleGlyphs->setMenuText( tr( "&Glyphs" ));
    actionToggleGlyphs->setStatusTip( tr( "Toggles the visibility of Glyphs" ) );
    actionToggleGrid->setText( tr( "Grid" ) );
    actionToggleGrid->setMenuText( tr( "G&rid" ) );
    actionToggleGrid->setStatusTip( tr( "Toggles the grid visibility" ) );
    actionTogglePlanes->setText( tr( "Planes" ) );
    actionTogglePlanes->setMenuText( tr( "&Cutting Planes" ) );
    actionTogglePlanes->setStatusTip( tr( "Toggles the visibility of cutting planes" ) );
    actionToggleIsosurface->setText( tr( "Isosurface") );
    actionToggleIsosurface->setMenuText( tr("&Isosurface" ) );
    actionToggleIsosurface->setStatusTip( tr( "Toggles the sosurface visibility" ) );
    actionGlyphProperties->setText( tr("Glyph"));
	actionGlyphProperties->setMenuText( tr("G&lyph..."));
	actionGlyphProperties->setStatusTip( tr("Sets the glyph properties"));
    actionIsoProperties->setText( tr( "Isosurface" ));
    actionIsoProperties->setMenuText( tr("&Isosurface..." ) );
    actionIsoProperties->setStatusTip( tr( "Sets the isosurface properties" ) );
    actionGridProperties->setText( tr( "Grid" ));
    actionGridProperties->setMenuText( tr( "&Grid...") );
    actionGridProperties->setStatusTip( tr( "Sets the grid properties" ) );
    actionPlanesProperties->setText( tr( "Planes" ) );
    actionPlanesProperties->setMenuText( tr("&Planes..." ) );
    actionPlanesProperties->setStatusTip( tr( "Sets the cutting planes properties" ) );
    actionPlaybackProperties->setText( tr( "Settings") );
    actionPlaybackProperties->setMenuText( tr( "&Settings..." ) );
    actionPlaybackProperties->setStatusTip( tr( "Sets the playback properties" ) );
    actionTogglePlay->setText( tr( "Play" ) );
    actionTogglePlay->setMenuText( tr( "&Play" ) );
    actionTogglePlay->setStatusTip( tr( "Toggles the frame playback" ) );
    actionTogglePlay->setAccel( tr( "Ctrl+P" ) );
    actionToggleRecord->setText( tr("Record"));
    actionToggleRecord->setMenuText(tr("&Record"));
    actionToggleRecord->setStatusTip(tr("Toggles playback recording"));
    actionToggleRecord->setAccel(tr("Ctrl+R"));
    actionHelp->setText( tr( "Contents" ) );
    actionHelp->setMenuText( tr( "&Contents" ) );
    actionHelp->setStatusTip( tr( "Opens the help file" ) );
    actionHelp->setAccel( tr( "F1" ) );
    actionAbout->setText( tr( "About" ) );
    actionAbout->setMenuText( tr( "&About" ) );
    actionAbout->setStatusTip( tr( "Displays information about capVTE" ) );
    actionToggleScalarGradient->setText( tr( "Scalar Gradient" ) );
    actionToggleScalarGradient->setMenuText( tr("&Scalar Gradient" ) );
    actionToggleScalarGradient->setStatusTip( tr("Toggles a scalar gradient represantation of the geometry") );
    toolbarFile->setLabel( tr( "File" ) );
    toolbarToggle->setLabel( tr( "Visibility" ) );
    toolbarPlay->setLabel( tr( "Play" ) );
    
	/*LB 2010: findItem doesn't work right with new menubar*/
	/*menubar->findItem( 0 )->setText( tr( "&File" ) );
    menubar->findItem( 1 )->setText( tr( "&View" ) );
    menubar->findItem( 2 )->setText( tr( "&Settings" ));
    menubar->findItem( 3 )->setText( tr( "&Playback" ));
    menubar->findItem( 4 )->setText( tr( "&Network" ));
    menubar->findItem( 5 )->setText( tr( "&Help" ) );*/

    actionToggleNetwork->setText(tr("Network"));
    actionToggleNetwork->setMenuText(tr("&Network Connection..."));
    actionToggleNetwork->setStatusTip(tr("Connects and disconnects the network"));

    actionToggleRedBlue->setText(tr("Red Blue 3D View"));
    actionToggleRedBlue->setMenuText(tr("&Red Blue"));
    actionToggleRedBlue->setStatusTip(tr("Toggles a Red Blue type 3D Visualization"));
    actionToggleCrystalEyes->setText(tr("Crystal Eyes 3D View"));
    actionToggleCrystalEyes->setMenuText(tr("&Crystal Eyes"));
    actionToggleCrystalEyes->setStatusTip(tr("Toggles a Crystal Eyes device 3D Visualization"));
    actionToggleNormalView->setText(tr("Normal View"));
    actionToggleNormalView->setMenuText(tr("&Normal View"));
    actionToggleNormalView->setStatusTip(tr("Return to Normal View"));
}  

//**FILE MENU**

//OpenDataset
void vteGUI::OpenDataset()
{
	QString filter("capVTE files (*.vte);;All files (*)");
	QStatusBar *status_bar=statusBar();

	QString filename = QFileDialog::getOpenFileName(QString::null,filter, this );
	if ( !filename.isEmpty() )
		{
			status_bar->message("loading...");
			vteGUIWindow *w=new vteGUIWindow(workspace, this, status_bar,filename, Qt::WStyle_NormalBorder | Qt::WDestructiveClose );
			connect( w, SIGNAL( message(const QString&, int)), statusBar(), SLOT( message(const QString&, int )) );

			w->setCaption(filename);
			w->setIcon( QPixmap(icocapVTE));
			w->show();
			
			w->resize(300,300);
			w->move(100,100);

			connect(w,SIGNAL(closeSignal()),this,SLOT(DisableToolbar()));
			connect(workspace,SIGNAL(subWindowActivated(QMdiSubWindow*)),this,SLOT(ActiveWindowChanged(QMdiSubWindow *)));
	
			ActiveWindowChanged(w);
			//status_bar->clear();
		}
}

void vteGUI::SaveDataset()
{
 QString jpeg_filter("JPEG files (*.jpg)");
 QString iv_filter("Inventor 2.0 files (*.iv)");
 QString vrml_filter("VRML 2.0 files (*.wrl)");
 QString sep(";;");
 QString selected_filter;

 QString filename = QFileDialog::getSaveFileName(QString::null,
            jpeg_filter+sep+iv_filter+sep+vrml_filter,this, 0, QString::null, &selected_filter );

 if ( !filename.isEmpty() )
		{
		active_window->repaint(TRUE);
    statusBar()->message("saving...");
    if (selected_filter==jpeg_filter)
				active_window->GetVisualizationWindow()->WriteJPEG((char *)filename.ascii());
		else if(selected_filter==iv_filter)
				active_window->GetVisualizationWindow()->WriteIV((char *)filename.ascii());
		else if(selected_filter==vrml_filter)
				active_window->GetVisualizationWindow()->WriteVRML((char *)filename.ascii());
		else qWarning("unknown file type");

		statusBar()->clear();
		}
}

void vteGUI::Close()
{
   qWarning("about to active_window->close()");
   active_window->close();
   qWarning("back");
}

void vteGUI::ToggleGeometry()
{
	if(!toggle_action_disabled)
	{
		if (active_window->IsNetworkConnected())
			active_window->GetNetwork()->PropagateMessage(tr("VTE TOGGLE_GEOMETRY"));

		if (active_window->GetVisualizationWindow()->ToggleGeometry()!=vteOK)
		{
			toggle_action_disabled=true;
			actionToggleGeometry->toggle();
			toggle_action_disabled=false;
		} 
 
		/*redraw image*/
		active_window->GetVisualizationWindow()->Repaint();
	}
}

void vteGUI::ToggleScalarGradient()
{
	if(!toggle_action_disabled)
	{
		if (active_window->IsNetworkConnected())
			active_window->GetNetwork()->PropagateMessage(tr("VTE TOGGLE_SCALAR_GRADIENT"));

		if (active_window->GetVisualizationWindow()->ToggleScalarGradient()!=vteOK)
		{
			toggle_action_disabled=true;
			actionToggleScalarGradient->toggle();
			toggle_action_disabled=false;
		}

		/*redraw image*/
		active_window->GetVisualizationWindow()->Repaint();
  }
}

void vteGUI::ToggleGrid()
{
	if (!toggle_action_disabled)
	{
		if (active_window->IsNetworkConnected())
		active_window->GetNetwork()->PropagateMessage(tr("VTE TOGGLE_GRID"));

		if (active_window->GetVisualizationWindow()->ToggleGrid()!=vteOK)
		{
			toggle_action_disabled=true;
			actionToggleGrid->toggle();
			toggle_action_disabled=false;
		} 
		
		/*redraw image*/
		active_window->GetVisualizationWindow()->Repaint();
	}
}

void vteGUI::ToggleGlyphs()
{
	if (!toggle_action_disabled)
	{
		if (active_window->IsNetworkConnected())
		active_window->GetNetwork()->PropagateMessage(tr("VTE TOGGLE_GLYPHS"));

		if (active_window->GetVisualizationWindow()->ToggleGlyphs()!=vteOK)
		{
			toggle_action_disabled=true;
			actionToggleGlyphs->toggle();
			toggle_action_disabled=false;
		} 
		
		/*redraw image*/
		active_window->GetVisualizationWindow()->Repaint();
	}
}

void vteGUI::TogglePlanes()
{
	if (!toggle_action_disabled)
	{
		if (active_window->IsNetworkConnected())
		active_window->GetNetwork()->PropagateMessage(tr("VTE TOGGLE_PLANES"));

		if (active_window->GetVisualizationWindow()->TogglePlanes()!=vteOK)
		{
			toggle_action_disabled=true;
			actionTogglePlanes->toggle();
			toggle_action_disabled=false;
		}

		/*redraw image*/
		active_window->GetVisualizationWindow()->Repaint();
	}
}

void vteGUI::ToggleIsosurface()
{
	if (!toggle_action_disabled)
	{
		if (active_window->IsNetworkConnected())
		active_window->GetNetwork()->PropagateMessage(tr("VTE TOGGLE_ISOSURFACE"));

		if (active_window->GetVisualizationWindow()->ToggleIsosurface()!=vteOK)
		{
			toggle_action_disabled=true;
			actionToggleIsosurface->toggle();
			toggle_action_disabled=false;
		}

		/*redraw image*/
		active_window->GetVisualizationWindow()->Repaint();
	}
}

void vteGUI::TogglePlay()
{
	if (!toggle_action_disabled)
	{
		if (active_window->IsNetworkConnected())
		active_window->GetNetwork()->PropagateMessage(tr("VTE TOGGLE_PLAY"));

		if (active_window->TogglePlay()!=vteOK)
		{
			toggle_action_disabled=true;
			actionTogglePlay->toggle();
			toggle_action_disabled=false;
		} 
	
		//no  need to repaint
	}
}

void vteGUI::ToggleRecord()
{
	if (!toggle_action_disabled)
	{
		if (active_window->ToggleRecord()!=vteOK)
		{
			toggle_action_disabled=true;
			actionToggleRecord->toggle();
			toggle_action_disabled=false;
		}
	}
}

//********
void vteGUI::ToggleNetwork()
{
	if (!toggle_action_disabled)
	{
		if (active_window->IsNetworkConnected()==true)
			active_window->StopNetwork();
		else
		{
			//start a new network connection
			vteDialogNetworkStart *dialog=new vteDialogNetworkStart(this);
			if (dialog->exec()==QDialog::Accepted)
			{
				bool server;
				long port;
				QString name;
				QString address;
				int speed;

				if (dialog->rbServer->isChecked()) server=true; else server=false;
				port=dialog->lePort->text().toLong();
				name=dialog->leName->text();
				speed=dialog->leConnectionSpeed->currentItem();
				address=dialog->leServerAddress->text();  //this will be ignored if server is true
	
				switch(speed)
				{
					case 0:speed=500;break;
					case 1:speed=200;break;
					case 2:speed=100;break;
					case 3:speed=50;break;
				}

				/*LB 2010: removed with switch from qtWindow to vteGUIWindow*/
				active_window->StartNetwork(port,name,"head1.vte",server,address);
				
			}
		}  //connected?
	}
}

void vteGUI::ToggleRedBlue()
{
	if (!toggle_action_disabled)
	{
		active_window->GetVisualizationWindow()->GetWindow()->SetStereoTypeToAnaglyph();		/*was RedBlue*/
		active_window->GetVisualizationWindow()->GetWindow()->StereoRenderOn();
		active_window->GetVisualizationWindow()->Repaint();
	}
}

void vteGUI::ToggleCrystalEyes()
{
	if (!toggle_action_disabled)
	{
		active_window->GetVisualizationWindow()->GetWindow()->StereoCapableWindowOn();
		active_window->GetVisualizationWindow()->GetWindow()->SetStereoTypeToCrystalEyes();
		active_window->GetVisualizationWindow()->GetWindow()->StereoRenderOn();
		active_window->GetVisualizationWindow()->Repaint();
	}
}

void vteGUI::ToggleNormalView()
{
	if (!toggle_action_disabled)
	{
		active_window->GetVisualizationWindow()->GetWindow()->StereoRenderOff();
		active_window->GetVisualizationWindow()->Render();
	}
}

void vteGUI::GetGlyphProperties()
{
	vteGlyphSetCollection *glyph_set = active_window->GetVisualizationWindow()->GetInput()->GetGlyphSetCollection();
	vtkGlyph3D *glyph = glyph_set->Get(0)->GetGlyphSetForVtk()->GetGlyph();
	double scaling_factor = glyph->GetScaleFactor();
	bool scaling;

	if (glyph->GetScaling()) scaling = true; else scaling=false;
	
	vteDialogGlyphProperties *dialog = new vteDialogGlyphProperties(scaling, scaling_factor,this);
      
	if (dialog->exec() == QDialog::Accepted)
	{
		int i;
		bool scaled=dialog->scalingBox->isChecked();
    
		scaling_factor=dialog->textScalingFactor->text().toFloat();
		for (i=0;i<glyph_set->GetNumberOfMembers();i++)
		{
			glyph=glyph_set->Get(i)->GetGlyphSetForVtk()->GetGlyph();
			double temp=glyph->GetScaleFactor();
			glyph->SetScaleFactor(scaling_factor);
			if (scaled) 
				glyph->ScalingOn(); 
			else 
				glyph->ScalingOff();
			glyph->Modified();
		}  
	}
	
	delete dialog;
}


void vteGUI::GetIsoProperties()
{
	vteVisualizationWindow *win=active_window->GetVisualizationWindow();
	unsigned int nol,i;
	double alpha;
	double *levels;
	unsigned int transparency;
	char message[36],NOL[6],ALPHA[6];
	QString text_levels;
	QTextStream ts(&text_levels,QIODevice::ReadWrite);

	vteIsosurfaceForVtk *isosurface_for_vtk=win->GetIsosurfaceForVtk();
	if (IsDefined(isosurface_for_vtk)==false) return;
 
	transparency = (int)((1.0-isosurface_for_vtk->GetAlpha())*100);
	nol = isosurface_for_vtk->GetNumberOfLevels();
	levels = isosurface_for_vtk->GetLevels();
	
	ts.setRealNumberPrecision(2);
	for (i=0;i<nol;i++) 
		ts<<levels[i]<<" ";

	vteDialogIsoProperties *dialog =
		new vteDialogIsoProperties(active_window->GetIsoPropertiesAuto(),nol,text_levels,transparency,this);
		
	if (dialog->exec()==QDialog::Accepted)
	{
		//auto or manual?
		if (dialog->radioAuto->isChecked())
		{
			active_window->SetIsoPropertiesAuto(true);
			nol=dialog->spinContourLevels->value();
			isosurface_for_vtk->GenerateLevels(nol);
        
			if(active_window->IsNetworkConnected())
			{
				gcvt((float)nol,3,NOL);
				strcpy(message,"VTE ISO AUTO ");
				strcat(message,NOL);
			}
		}
		else
		{
			QString text_levels_back;
			active_window->SetIsoPropertiesAuto(false);
			text_levels_back=dialog->textLevels->text();
			text_levels_back.simplifyWhiteSpace();

			QTextStream ts_back (&text_levels_back,QIODevice::ReadOnly);

			levels=new double[300];
			if (levels==NULL) {ErrorMessage("insufficient memory for levels"); return;}

			nol=0;
			qWarning(text_levels_back);
  			while(!ts_back.atEnd() && nol<300)
				{
					ts_back>>levels[nol];
					ts_back.skipWhiteSpace();
					nol++;
				}
         
			if (nol>0) isosurface_for_vtk->SetContours(nol,levels);
			if(active_window->IsNetworkConnected())
			{
				gcvt((float)nol,5,NOL);
				strcpy(message,"VTE ISO MANUAL ");
				strcat(message,NOL);
				for (i=0;i<nol;i++)
				{
					gcvt((float)levels[i],8,NOL);
					strcat(message," ");
					strcat(message,NOL);
				}
			}
			
			delete levels;
       }
    
    alpha=(100.0-(double)dialog->spinTransparency->value())/100.0;
    isosurface_for_vtk->SetAlpha(alpha);

    if (active_window->IsNetworkConnected())
    {
      strcat(message," ");
      gcvt(alpha,5,ALPHA);
      strcat(message,ALPHA);
      active_window->GetNetwork()->PropagateMessage(tr(message));
      }
    }
//repaint
 active_window->repaint(TRUE);
 delete dialog;
}

//////*******************
void vteGUI::GetPlaneProperties()
{
	vteVisualizationWindow *visualization_window=active_window->GetVisualizationWindow();
	vteCuttingPlaneCollection *cutting_plane_collection=visualization_window->GetCuttingPlaneCollection();
	vteGridForVtk *grid_for_vtk=visualization_window->GetInput()->GetGridFrameCollection()->GetGridForVtk();

	if (IsDefined(cutting_plane_collection)==false) return;

	vtePlaneDefinition *plane_definition=new vtePlaneDefinition[cutting_plane_collection->GetNumberOfMembers()];
	if (plane_definition==NULL) return;

	int i,nop;
	vteCuttingPlane *cutting_plane;
	nop=cutting_plane_collection->GetNumberOfMembers();
	vtePoint *origin;
	vteVector *normal;

	for (i=0;i<nop;i++)
	{
		cutting_plane=cutting_plane_collection->GetCuttingPlane(i);
		origin=cutting_plane->GetOrigin();
		normal=cutting_plane->GetNormal();
		VEC_COPY(plane_definition[i].origin.x,origin->x);
		VEC_COPY(plane_definition[i].normal.u,normal->u);

		plane_definition[i].number_of_levels=cutting_plane->GetFilter()->GetNumberOfContours();
		plane_definition[i].alpha=cutting_plane->GetCutterActor()->GetProperty()->GetOpacity();
	}

	vteDialogPlanesProperties *dialog=new vteDialogPlanesProperties(plane_definition,nop,this);

	if (dialog->exec()==QDialog::Accepted)
	{
		int nop_dialog=dialog->listPlanes->model()->rowCount();
		bool planes_visible=visualization_window->GetPlanesVisibility();

		QString string;
		QTextStream message(&string);
		message<<"VTE PLANES "<<nop_dialog;
		              
		//remove all plane actors
		if (planes_visible) visualization_window->TogglePlanes();

		if (nop_dialog>nop)
			for (i=nop;i<nop_dialog;i++) cutting_plane_collection->NewCuttingPlane(grid_for_vtk);
		else if (nop_dialog<nop)
			for (i=nop_dialog;i<nop;i++) cutting_plane_collection->RemoveCuttingPlane(i);

		vtePoint point;
	    vteVector vector;
		vteRange *range=visualization_window->GetInput()->GetGridFrameCollection()->GetUserRange();

		for (i=0;i<nop_dialog;i++)
		{
			VEC_COPY(point.x,plane_definition[i].origin.x);
			VEC_COPY(vector.u,plane_definition[i].normal.u);

			message<<point.x[0]<<" "<<point.x[1]<<" "<<point.x[2]<<" ";
			message<<vector.u[0]<<" "<<vector.u[1]<<" "<<vector.u[2]<<" ";
			message<<plane_definition[i].number_of_levels<<" ";
			message<<plane_definition[i].alpha<<" ";
			
			cutting_plane=cutting_plane_collection->GetCuttingPlane(i);
			cutting_plane->GetPlane()->SetOrigin(point.x);
			cutting_plane->GetPlane()->SetNormal(vector.u);

			cutting_plane->GetFilter()->GenerateValues(plane_definition[i].number_of_levels,range->min,range->max);
			//cutting_plane->GetCutter()->GenerateValues(plane_definition[i].number_of_levels, range->min,range->max);
			cutting_plane->GetCutterActor()->GetProperty()->SetOpacity(plane_definition[i].alpha);
			cutting_plane->GetContourActor()->GetProperty()->SetOpacity(plane_definition[i].alpha);
			cutting_plane->SetOrigin(&point);
			cutting_plane->SetNormal(&vector);
			cutting_plane->GetCutter()->Modified();
			cutting_plane->GetPlane()->Modified();
		}

		//show planes
		if (planes_visible) visualization_window->TogglePlanes();
		if (active_window->IsNetworkConnected()) active_window->GetNetwork()->PropagateMessage(string);
    }

	active_window->repaint(TRUE);
	delete dialog;
}

//---**GRID***
void vteGUI::GetGridProperties()
{
	vteVisualizationWindow *win=active_window->GetVisualizationWindow();
	QString *names;
	vteRange *absolute_range, *user_range;
	vteGridFrameCollection *frame;
	vteGridForVtk *grid_for_vtk;
	vteIsosurfaceForVtk *isosurface;
	vteScalarBarForVtk *scalar_bar;
	//vteAxesForVtk *axes;

	char message[50],COMPONENT[8];
	long noc,active_component,i;

	grid_for_vtk=win->GetInput()->GetGridFrameCollection()->GetGridForVtk();
	isosurface=win->GetIsosurfaceForVtk();
	scalar_bar=win->GetScalarBarForVtk();
	frame=grid_for_vtk->GetInput();
	noc=frame->GetNumberOfVars();
	active_component=frame->GetActiveScalar();
	//axes=win->GetInput()->GetAxesForVtk();

	bool gradient_visible=win->GetScalarGradientVisibility();

	absolute_range=grid_for_vtk->GetAbsoluteRange(0);
	user_range=grid_for_vtk->GetUserRange(0);

	names=new QString[noc];
	for (i=0;i<noc;i++)
		names[i]=frame->GetVarName(i);
	
	vteDialogGridProperties *dialog=new vteDialogGridProperties(noc,active_component,absolute_range,user_range,names,this);

	if (dialog->exec()==QDialog::Accepted)
	{
		//to be cleaned later on...
		active_component = dialog->listComponent->selectionModel()->selectedRows().first().row(); /*FIX THIS*/
		
		vtkRenderer *renderer = active_window->GetVisualizationWindow()->GetRenderer();
		vteVisualizationWindow *vis = active_window->GetVisualizationWindow();
		vteGridForVtk *grid_for_vtk = vis->GetInput()->GetGridFrameCollection()->GetGridForVtk();
  
		frame->SetActiveVar(active_component);
		
		active_window->GetVisualizationWindow()->UpdateToFrame(frame->GetCurrentFrame());

		/*get new user range*/
		double min, max;
		min = atof(dialog->textMin->text());
		max = atof(dialog->textMax->text());

		/*update user range, this will also update LUT*/
		vis->GetInput()->GetGridFrameCollection()->SetUserRange(min, max);

		vteCuttingPlaneCollection *plane_collection = vis->GetCuttingPlaneCollection();
		for (int i=0; i<plane_collection->GetNumberOfMembers();i++)
			vis->GetCuttingPlaneCollection()->GetCuttingPlane(i)->Modified();

		vis->GetIsosurfaceForVtk()->GetActor()->Modified();
				
		vis->GetScalarBarForVtk()->Update();
		
		if (active_window->IsNetworkConnected())
		{
			strcpy(message,"VTE GRID ");
			gcvt((float)active_component,7,COMPONENT);
			strcat(message,COMPONENT);
			active_window->GetNetwork()->PropagateMessage(message);
		}
	}
      

	active_window->repaint(TRUE);
	delete dialog;
	//delete names;
}

//---**PLAYBACK**
void vteGUI::GetPlaybackProperties()
{
 char message[25],RATE[4],FRAME[5];
 vteVisualizationWindow *win=active_window->GetVisualizationWindow();
 vteDialogPlaybackProperties *dialog=new vteDialogPlaybackProperties(win->GetCurrentFrame(),win->GetMaxFrame(),active_window->GetPlaybackRate(),this);

 if (dialog->exec()==QDialog::Accepted)
		{
     active_window->SetPlaybackRate(dialog->sliderRate->value());
	   win->UpdateToFrame(dialog->sliderFrame->value()-1);    //frames are shown as going from 1 to Max+1
     if (active_window->IsNetworkConnected())
     {
       strcpy(message,"VTE PLAYBACK ");
       gcvt((float)dialog->sliderRate->value(),4,RATE);
       strcat(message,RATE);
       strcat(message," ");
       gcvt((float)dialog->sliderFrame->value(),5,FRAME);
       strcat(message,FRAME);
       active_window->GetNetwork()->PropagateMessage(tr(message));
     }
    }

 delete dialog;
}

void vteGUI::ShowHelp()
{
	QString path;

	path=QDir("doc/index.html").absPath();

	vteHelpWindow *help = new vteHelpWindow(path,this);
	help->show();
}

void vteGUI::About()
{
	QString caption;
	QTextStream ts (&caption,QIODevice::WriteOnly);

	ts<<"About capVTE "<<VERSION<<" (BETA)";
	QMessageBox::information(this,caption,"(c) 2010 Lubos Brieda\n"
		"Particle In Cell Consulting, http://www.particleincell.com\n\n"
		"Original version created in collaboration with Julien Pierru,\n"
		"with assistance from Binh Tran, Joseph Wang and Raed Kafafy\n\n"
		"This program uses VTK (vtk.org) QT (qt.nokia.com)");
}

//enables and activates toolbar toggle options
void vteGUI::ActiveWindowChanged(QMdiSubWindow *w)
{
	if (!w || (active_window == w)) return;  //do nothing

	toggle_action_disabled=true;

	active_window=(vteGUIWindow*)workspace->activeSubWindow();
	if (!active_window)
		active_window=(vteGUIWindow*)w;

	vteVisualizationWindow *win=active_window->GetVisualizationWindow();

	actionToggleGeometry->setEnabled(win->HasGeometry()?TRUE:FALSE);
	actionToggleGlyphs->setEnabled(win->HasGlyphs()?TRUE:FALSE);
	actionToggleGrid->setEnabled(win->HasGrid()?TRUE:FALSE);
	actionToggleScalarGradient->setEnabled((win->HasGrid() && win->HasGeometry())?TRUE:FALSE);
	actionTogglePlanes->setEnabled(win->HasGrid()?TRUE:FALSE);
	actionToggleIsosurface->setEnabled(win->HasGrid()?TRUE:FALSE);
	actionTogglePlay->setEnabled(win->HasFrames()?TRUE:FALSE);
	actionToggleRecord->setEnabled(TRUE);
	actionPlaybackProperties->setEnabled(actionTogglePlay->isEnabled());

	actionIsoProperties->setEnabled(win->HasGrid()?TRUE:FALSE);
	actionGridProperties->setEnabled(win->HasGrid()?TRUE:FALSE);
	actionPlanesProperties->setEnabled(win->HasGrid()?TRUE:FALSE);
	actionGlyphProperties->setEnabled(win->HasGlyphs()?TRUE:FALSE);

	actionToggleGeometry->setOn((win->HasGeometry() && win->GetGeometryVisibility())?TRUE:FALSE);
	actionToggleGlyphs->setOn((win->HasGlyphs() && win->GetGlyphsVisibility())?TRUE:FALSE);
	actionToggleGrid->setOn((win->HasGrid() && win->GetGridVisibility())?TRUE:FALSE);
	actionToggleScalarGradient->setOn((win->HasGrid() && win->HasGeometry() && 
	win->GetScalarGradientVisibility())?TRUE:FALSE);
	actionTogglePlanes->setOn((win->HasGrid() && win->GetPlanesVisibility())?TRUE:FALSE);
	actionToggleIsosurface->setOn((win->HasGrid() && win->GetIsosurfaceVisibility())?TRUE:FALSE);
	actionTogglePlay->setOn(active_window->IsPlaying()?TRUE:FALSE);
	actionTogglePlay->setOn(active_window->IsRecording()?TRUE:FALSE);

	/*for now turn off network*/
	actionToggleNetwork->setEnabled(FALSE);
	actionToggleNetwork->setOn(active_window->IsNetworkConnected());

	actionToggleRedBlue->setEnabled(TRUE);
	actionToggleCrystalEyes->setEnabled(TRUE);
	actionToggleNormalView->setEnabled(TRUE);
	
	toggle_action_disabled=false;
}

void vteGUI::DisableToolbar()
{
	toggle_action_disabled=true;
	actionToggleGeometry->setEnabled(FALSE);
	actionToggleGlyphs->setEnabled(FALSE);
	actionToggleGrid->setEnabled(FALSE);
	actionToggleScalarGradient->setEnabled(FALSE);
	actionTogglePlanes->setEnabled(FALSE);
	actionToggleIsosurface->setEnabled(FALSE);
	actionTogglePlay->setEnabled(FALSE);
	actionToggleRecord->setEnabled(FALSE);
	actionPlaybackProperties->setEnabled(FALSE);

	actionIsoProperties->setEnabled(FALSE);
	actionGridProperties->setEnabled(FALSE);
	actionPlanesProperties->setEnabled(FALSE);
	actionGlyphProperties->setEnabled(FALSE);

	actionToggleGeometry->setOn(FALSE);
	actionToggleGlyphs->setOn(FALSE);
	actionToggleGeometry->setOn(FALSE);
	actionTogglePlanes->setOn(FALSE);
	actionToggleScalarGradient->setOn(FALSE);
	actionToggleIsosurface->setOn(FALSE);
	actionTogglePlay->setOn(FALSE);

	actionToggleNetwork->setEnabled(FALSE);
	actionToggleNetwork->setOn(FALSE);

	actionToggleRedBlue->setEnabled(FALSE);
	actionToggleRedBlue->setOn(FALSE);
	actionToggleCrystalEyes->setEnabled(FALSE);
	actionToggleCrystalEyes->setOn(FALSE);
	actionToggleNormalView->setEnabled(FALSE);
	actionToggleNormalView->setOn(FALSE);

	toggle_action_disabled=false;
}