#include "vteGlyphs.h"
#include "vteGlyphs.h"

#include "vtkSphereSource.h"
#include "vtkConeSource.h"
#include "vtkCubeSource.h"
#include "vtkPointSource.h"
#include "vtkPolyData.h"
#include "vtkPolyDataMapper.h"
#include "vtkUnsignedIntArray.h"
#include "vtkDoubleArray.h"
#include "vtkPointData.h"

#include <iostream>
using namespace std;

/***********************
vteGlyphSetCollection::vteGlyphSetCollection

Constructor
************************/
vteGlyphSetCollection::vteGlyphSetCollection()
{
	glyph_frame_collection = NULL;
}

/***********************
vteGlyphSetCollection::~vteGlyphSetCollection

Destructor
************************/
vteGlyphSetCollection::~vteGlyphSetCollection()
{
	for (int i=0;i<members;i++)
		delete glyph_frame_collection[i];

	delete glyph_frame_collection;
}

/*************************************************
vteGlyphSetCollection::AddCollection

Adds another glyph frame collection to the list
**************************************************/
vteGlyphFrameCollection* vteGlyphSetCollection::AddNew()
{
	if (glyph_frame_collection)
		glyph_frame_collection = (vteGlyphFrameCollection**) realloc(glyph_frame_collection, (members+1)*sizeof(vteGlyphFrameCollection*));
	else
		glyph_frame_collection = new (vteGlyphFrameCollection*);

	glyph_frame_collection[members] = new vteGlyphFrameCollection;
	members++;

	return glyph_frame_collection[members-1];
}

/*************************************************
vteGlyphSetCollection::GetCollection

Retrieves specified collection from the list
**************************************************/
vteGlyphFrameCollection* vteGlyphSetCollection::Get(int i)
{
	if (i<0 || i>=members) return NULL;

	return glyph_frame_collection[i];
}

/*******************************************
vteGlyphFrameCollection

GlyphFrameCollection is a collection of glyph frames, 
each containing arbitrary number of glyphs
********************************************/
vteGlyphFrameCollection::vteGlyphFrameCollection()
{
	shape = NULL;
	frame = NULL;
	number_of_shapes=0;
	glyph_set_for_vtk=NULL;
}

vteGlyphFrameCollection::~vteGlyphFrameCollection()
{
	for (int i=0;i<members;i++)
		delete frame[i];
	delete frame;
	
	delete shape;
}

vteGlyphFrame* vteGlyphFrameCollection::GetFrame(long index)
{
	if (index<0 || index>=max_frame)
		return frame[frame_index];

	return frame[index];
}

vteGlyphFrame* vteGlyphFrameCollection::AddFrame(int nog)
{
	if (frame)
		frame = (vteGlyphFrame**) realloc(frame, (members+1)*sizeof(vteGlyphFrame*));
	else
		frame = new (vteGlyphFrame*);

	frame[members] = new vteGlyphFrame(nog);
	members++;
	max_frame = members-1;		/*update max_frame*/

	return frame[members-1];
}

unsigned int vteGlyphFrameCollection::MembersInFrame(int index)
{
	if (index<0)
		return frame[frame_index]->GetNumberOfMembers();
	else
		return frame[index]->GetNumberOfMembers();
}

unsigned int vteGlyphFrameCollection::MaxMembers()
{
	unsigned int max_members = 0;
	for (int i=0;i<members;i++)
		if (MembersInFrame(i)>max_members) max_members=MembersInFrame(i);

	return max_members;
}

int vteGlyphFrameCollection::AddShape(GlyphSource source,double size,vteColor color)
{
	if (!shape)
		shape = new vteGlyphGeometry;
	else
		shape = (vteGlyphGeometry*) realloc(shape, sizeof(vteGlyphGeometry)*(number_of_shapes+1));

	shape[number_of_shapes].source = source;
	shape[number_of_shapes].size = size;
	shape[number_of_shapes].color = color;

	number_of_shapes++;
	return vteOK;
}

vteGlyphSetForVtk *vteGlyphFrameCollection::GetGlyphSetForVtk()
{
	if (!glyph_set_for_vtk)
		glyph_set_for_vtk = vteGlyphSetForVtk::New(this);
	return glyph_set_for_vtk;
}

/*****************************************
vteGlyphFrame

GlyphFrame contains info about all glyphs 
existing in a particular frame. 
******************************************/
vteGlyphFrame::vteGlyphFrame(int number_of_glyphs)
{
	members = 0;
	max_members = number_of_glyphs;
	glyphs = new vteGlyph[number_of_glyphs];		/*shoudl do something here*/
}

vteGlyphFrame::~vteGlyphFrame()
{
	delete glyphs;
}

int vteGlyphFrame::AddGlyph(vteGlyph *glyph)
{
	if (members<max_members)
		glyphs[members++]=*glyph;

	return vteOK;
}



/*****************************
VTK stuff
******************************/
/*to do:
 move initialization of generals source from vteGlyphSetForVtk to vteGeneralSource
*/

vteGeneralSource::vteGeneralSource()
{
	sphere_source=NULL;
	cone_source=NULL;
	cube_source=NULL;
	point_source=NULL;
	output=NULL;
}

vteGeneralSource::~vteGeneralSource()
{
	if (sphere_source!=NULL) sphere_source->Delete();
	if (cone_source!=NULL) cone_source->Delete();
	if (cube_source!=NULL) cube_source->Delete();
	if (point_source!=NULL) point_source->Delete();
}

//----------------------------
vteGlyphSetForVtk::vteGlyphSetForVtk()
{
	actor=NULL;
	source=NULL;
	mapper=NULL;
	glyph=NULL;
	grid=NULL;
	lut=NULL;
	filter=NULL;
	input=NULL;
	number_of_sources=0;
}

vteGlyphSetForVtk::~vteGlyphSetForVtk()
{
	unsigned int i;
	if (actor!=NULL) actor->Delete();
	
	for (i=0;i<number_of_sources;i++)
		if (source[i]!=NULL) delete source[i];
	
	if (mapper!=NULL) mapper->Delete();
	if (glyph!=NULL) glyph->Delete();
	if (grid!=NULL) grid->Delete();
	if (lut!=NULL) lut->Delete();
	if (filter!=NULL) filter->Delete();
}

vteGlyphSetForVtk *vteGlyphSetForVtk::New(vteGlyphFrameCollection *frames)
{
	int i;
	double v[3];
	double rgba[4];

	if (IsDefined(frames)==false) return NULL;

	vteGlyphSetForVtk *glyph_set_for_vtk = new vteGlyphSetForVtk;
	if (glyph_set_for_vtk==NULL) return NULL;

	//generate grid used to hold glyph positions
	vteGlyph *glyph;
	int number_of_glyphs;
	int max_glyphs;

	glyph=frames->GetFrame()->GetGlyphs();

	number_of_glyphs=frames->MembersInFrame();
	max_glyphs=frames->MaxMembers();

	vtkPoints *nodes=vtkPoints::New();
	if (nodes==NULL) return NULL;
	nodes->SetNumberOfPoints(max_glyphs);

	vtkUnsignedIntArray *types=vtkUnsignedIntArray::New();
	if (types==NULL) return NULL;
	types->SetNumberOfValues(max_glyphs);

	vtkDoubleArray *vectors=vtkDoubleArray::New();
	if (vectors==NULL) return NULL;
	vectors->SetNumberOfComponents(3);
	vectors->SetNumberOfTuples(max_glyphs);

	for (i=0;i<number_of_glyphs;i++)
	{
		nodes->SetPoint(i,glyph[i].x);
		types->SetValue(i,glyph[i].t);
		vectors->SetTuple(i,glyph[i].u);
	}

	//fill the rest with zeros
	for (i=number_of_glyphs;i<max_glyphs;i++)
	{
		nodes->SetPoint(i,0,0,0);
		types->SetValue(i,0);
		v[0]=v[1]=v[2]=0;
		vectors->SetTuple(i,v);
	}

	glyph_set_for_vtk->grid=vtkStructuredGrid::New();
	if (glyph_set_for_vtk->grid==NULL) return NULL;
	glyph_set_for_vtk->grid->SetDimensions(max_glyphs,1,1);
	glyph_set_for_vtk->grid->SetPoints(nodes);
	 nodes->Delete();
	glyph_set_for_vtk->grid->GetPointData()->SetScalars(types);
	types->Delete();
	glyph_set_for_vtk->grid->GetPointData()->SetVectors(vectors);
	vectors->Delete();
	
	//LB port
	// glyph_set_for_vtk->grid->BlankingOff();

	//allocate memory for sources and glyph3d
	glyph_set_for_vtk->source=new vteGeneralSource*[frames->GetNumberOfShapes()];
	if (glyph_set_for_vtk->source==NULL) return NULL;

	for (i=0;i<frames->GetNumberOfShapes();i++)
	{
		glyph_set_for_vtk->source[i]=new vteGeneralSource;
		if (glyph_set_for_vtk->source[i]==NULL) return NULL;
	}

	glyph_set_for_vtk->number_of_sources=frames->GetNumberOfShapes();

	glyph_set_for_vtk->glyph=vtkGlyph3D::New();
	if (glyph_set_for_vtk->glyph==NULL) return NULL;
 
	glyph_set_for_vtk->filter=vtkStructuredGridGeometryFilter::New();
	glyph_set_for_vtk->filter->SetInput(glyph_set_for_vtk->grid);
	glyph_set_for_vtk->filter->SetExtent(0,number_of_glyphs-1,0,1,0,1);

	//LB 
	//glyph_set_for_vtk->glyph->SetNumberOfSources(glyph_set_for_vtk->number_of_sources);
	glyph_set_for_vtk->glyph->ScalingOff();
	glyph_set_for_vtk->glyph->SetScaleModeToScaleByVector();
	glyph_set_for_vtk->glyph->SetIndexModeToScalar();
	glyph_set_for_vtk->glyph->SetColorModeToColorByScalar();
	glyph_set_for_vtk->glyph->SetVectorModeToUseVector();
	glyph_set_for_vtk->glyph->OrientOn();
	glyph_set_for_vtk->glyph->SetRange(glyph_set_for_vtk->grid->GetScalarRange());
	glyph_set_for_vtk->glyph->SetInput(glyph_set_for_vtk->filter->GetOutput());

	vteGlyphGeometry *geom;
	//now generate sources
	for (unsigned int i=0;i<glyph_set_for_vtk->number_of_sources;i++)
	{
		geom=frames->GetSource(i);
					
		switch(geom->source)
		{
			case CLOUD: glyph_set_for_vtk->source[i]->point_source=vtkPointSource::New();
						if (glyph_set_for_vtk->source[i]->point_source==NULL) return NULL;
						glyph_set_for_vtk->source[i]->point_source->SetRadius(geom->size);
						glyph_set_for_vtk->source[i]->point_source->SetNumberOfPoints(25);
						glyph_set_for_vtk->source[i]->output=glyph_set_for_vtk->source[i]->point_source->GetOutput();
						break;
 			case TRIANGLE: glyph_set_for_vtk->source[i]->cone_source=vtkConeSource::New();
						if (glyph_set_for_vtk->source[i]->cone_source==NULL) return NULL;
						glyph_set_for_vtk->source[i]->cone_source->SetResolution(1);
						glyph_set_for_vtk->source[i]->cone_source->SetRadius(geom->size);
						glyph_set_for_vtk->source[i]->cone_source->SetHeight(3*geom->size);
						glyph_set_for_vtk->source[i]->output=glyph_set_for_vtk->source[i]->cone_source->GetOutput();
						break;

			case WEDGE: glyph_set_for_vtk->source[i]->cone_source=vtkConeSource::New();
						if (glyph_set_for_vtk->source[i]->cone_source==NULL) return NULL;
						glyph_set_for_vtk->source[i]->cone_source->SetResolution(3);
						glyph_set_for_vtk->source[i]->cone_source->SetRadius(geom->size);
						glyph_set_for_vtk->source[i]->cone_source->SetHeight(3*geom->size);
						glyph_set_for_vtk->source[i]->output=glyph_set_for_vtk->source[i]->cone_source->GetOutput();
						break;

			case LO_RES_CONE: glyph_set_for_vtk->source[i]->cone_source=vtkConeSource::New();
							  if (glyph_set_for_vtk->source[i]->cone_source==NULL) return NULL;
							  glyph_set_for_vtk->source[i]->cone_source->SetResolution(4);
							  glyph_set_for_vtk->source[i]->cone_source->SetRadius(geom->size);
							  glyph_set_for_vtk->source[i]->cone_source->SetHeight(3*geom->size);
							  glyph_set_for_vtk->source[i]->output=glyph_set_for_vtk->source[i]->cone_source->GetOutput();
							  break;

			case MED_RES_CONE: glyph_set_for_vtk->source[i]->cone_source=vtkConeSource::New();
							   if (glyph_set_for_vtk->source[i]->cone_source==NULL) return NULL;
							   glyph_set_for_vtk->source[i]->cone_source->SetResolution(8);
							   glyph_set_for_vtk->source[i]->cone_source->SetRadius(geom->size);
							   glyph_set_for_vtk->source[i]->cone_source->SetHeight(3*geom->size);
							   glyph_set_for_vtk->source[i]->output=glyph_set_for_vtk->source[i]->cone_source->GetOutput();
							   break;

			case HI_RES_CONE: glyph_set_for_vtk->source[i]->cone_source=vtkConeSource::New();
							  if (glyph_set_for_vtk->source[i]->cone_source==NULL) return NULL;
							  glyph_set_for_vtk->source[i]->cone_source->SetResolution(16);
							  glyph_set_for_vtk->source[i]->cone_source->SetRadius(geom->size);
							  glyph_set_for_vtk->source[i]->cone_source->SetHeight(3*geom->size);
							  glyph_set_for_vtk->source[i]->output=glyph_set_for_vtk->source[i]->cone_source->GetOutput();
							  break;

  			case CUBE: glyph_set_for_vtk->source[i]->cube_source=vtkCubeSource::New();
					   if (glyph_set_for_vtk->source[i]->cube_source==NULL) return NULL;
					   glyph_set_for_vtk->source[i]->cube_source->SetXLength(2*geom->size);
					   glyph_set_for_vtk->source[i]->cube_source->SetYLength(2*geom->size);
					   glyph_set_for_vtk->source[i]->cube_source->SetZLength(2*geom->size);
					   glyph_set_for_vtk->source[i]->output=glyph_set_for_vtk->source[i]->cube_source->GetOutput();
  					   break;

			case LO_RES_SPHERE: glyph_set_for_vtk->source[i]->sphere_source=vtkSphereSource::New();
					        if (glyph_set_for_vtk->source[i]->sphere_source==NULL) return NULL;
							glyph_set_for_vtk->source[i]->sphere_source->SetPhiResolution(4);
							glyph_set_for_vtk->source[i]->sphere_source->SetThetaResolution(4);
							glyph_set_for_vtk->source[i]->sphere_source->SetRadius(geom->size);
							glyph_set_for_vtk->source[i]->output=glyph_set_for_vtk->source[i]->sphere_source->GetOutput();
							break;

			case MED_RES_SPHERE: glyph_set_for_vtk->source[i]->sphere_source=vtkSphereSource::New();
							     if (glyph_set_for_vtk->source[i]->sphere_source==NULL) return NULL;
								 glyph_set_for_vtk->source[i]->sphere_source->SetPhiResolution(7);
								 glyph_set_for_vtk->source[i]->sphere_source->SetThetaResolution(7);
								 glyph_set_for_vtk->source[i]->sphere_source->SetRadius(geom->size);
								 glyph_set_for_vtk->source[i]->output=glyph_set_for_vtk->source[i]->sphere_source->GetOutput();
								break;

			case HI_RES_SPHERE: glyph_set_for_vtk->source[i]->sphere_source=vtkSphereSource::New();
							    if (glyph_set_for_vtk->source[i]->sphere_source==NULL) return NULL;
								glyph_set_for_vtk->source[i]->sphere_source->SetPhiResolution(12);
								glyph_set_for_vtk->source[i]->sphere_source->SetThetaResolution(12);
								glyph_set_for_vtk->source[i]->sphere_source->SetRadius(geom->size);
								glyph_set_for_vtk->source[i]->output=glyph_set_for_vtk->source[i]->sphere_source->GetOutput();
								break;

			default: return NULL;
		}

		glyph_set_for_vtk->glyph->SetSource(i,glyph_set_for_vtk->source[i]->output);
	}

	//generate lookup table
	glyph_set_for_vtk->lut=vtkLookupTable::New();

	if (glyph_set_for_vtk->lut==NULL) return NULL;

	glyph_set_for_vtk->lut->SetNumberOfTableValues(glyph_set_for_vtk->number_of_sources);
	for (unsigned int i=0;i<glyph_set_for_vtk->number_of_sources;i++)
	{
		geom=frames->GetSource(i);
		rgba[0]=geom->color.r;
		rgba[1]=geom->color.g;
		rgba[2]=geom->color.b;
		rgba[3]=geom->color.a;
		glyph_set_for_vtk->lut->SetTableValue(i,rgba);
	}
	glyph_set_for_vtk->lut->SetTableRange(0,glyph_set_for_vtk->number_of_sources-1);

	//create mapper
	glyph_set_for_vtk->mapper=vtkPolyDataMapper::New();
	if (glyph_set_for_vtk->mapper==NULL) return NULL;
	glyph_set_for_vtk->mapper->SetInput(glyph_set_for_vtk->glyph->GetOutput());
	glyph_set_for_vtk->mapper->SetLookupTable(glyph_set_for_vtk->lut);
	glyph_set_for_vtk->mapper->SetScalarRange(glyph_set_for_vtk->grid->GetScalarRange());

	glyph_set_for_vtk->actor=vtkLODActor::New();
	if (glyph_set_for_vtk->actor==NULL) return NULL;
	glyph_set_for_vtk->actor->SetMapper(glyph_set_for_vtk->mapper);
	glyph_set_for_vtk->actor->SetNumberOfCloudPoints(45);

	glyph_set_for_vtk->input=frames->GetFrame();

	Define(glyph_set_for_vtk);
	return glyph_set_for_vtk;
}

void vteGlyphSetForVtk::Delete()
{
	delete this;
}
