#include "vteIsosurface.h"

vteIsosurfaceForVtk::vteIsosurfaceForVtk()
{
	mapper=NULL;
	actor=NULL;
	grid_for_vtk=NULL;
}

vteIsosurfaceForVtk::~vteIsosurfaceForVtk()
{
	if (mapper!=NULL) mapper->Delete();
	if (actor!=NULL) actor->Delete();
	if (sinc_filter!=NULL) sinc_filter->Delete();
}

vteIsosurfaceForVtk *vteIsosurfaceForVtk::New(vteGridForVtk *grid_for_vtk)
{
	vteIsosurfaceForVtk *isosurface_for_vtk=new vteIsosurfaceForVtk();
	if (isosurface_for_vtk==NULL) return NULL;

	isosurface_for_vtk->filter=vtkContourFilter::New();
	if (isosurface_for_vtk->filter==NULL) return NULL;
	isosurface_for_vtk->filter->SetInput(grid_for_vtk->GetStructuredGrid());
	isosurface_for_vtk->filter->GenerateValues(5,grid_for_vtk->GetUserRange()->min,grid_for_vtk->GetUserRange()->max);

	isosurface_for_vtk->sinc_filter=vtkWindowedSincPolyDataFilter::New();
	if (isosurface_for_vtk->sinc_filter==NULL) return NULL;
	isosurface_for_vtk->sinc_filter->SetInput(isosurface_for_vtk->filter->GetOutput());
	isosurface_for_vtk->sinc_filter->SetNumberOfIterations(50);

	isosurface_for_vtk->mapper=vtkPolyDataMapper::New();
	if (isosurface_for_vtk->mapper==NULL) return NULL;
	isosurface_for_vtk->mapper->SetInput(isosurface_for_vtk->sinc_filter->GetOutput());
	isosurface_for_vtk->mapper->SetScalarRange(grid_for_vtk->GetUserRange()->min,grid_for_vtk->GetUserRange()->max);
	isosurface_for_vtk->mapper->SetLookupTable(grid_for_vtk->GetLut());

	isosurface_for_vtk->actor=vtkLODActor::New();
	if (isosurface_for_vtk->actor==NULL) return NULL;

	isosurface_for_vtk->actor->SetMapper(isosurface_for_vtk->mapper);
	isosurface_for_vtk->actor->GetProperty()->SetOpacity(1);

	isosurface_for_vtk->grid_for_vtk=grid_for_vtk;
	Define(isosurface_for_vtk);
  
	return isosurface_for_vtk;
}

void vteIsosurfaceForVtk::Delete()
{
	delete this;
}

void vteIsosurfaceForVtk::SetAlpha(float a)
{
	actor->GetProperty()->SetOpacity(a);
	filter->Modified();
}

vteIsosurfaceForVtk *vteIsosurfaceForVtk::UpdateIsosurface(vteGridForVtk *grid_for_vtk)
{
	vteIsosurfaceForVtk *isosurface_for_vtk=new vteIsosurfaceForVtk();
	if (isosurface_for_vtk==NULL) return NULL;

	isosurface_for_vtk->filter=vtkContourFilter::New();
	if (isosurface_for_vtk->filter==NULL) return NULL;
	isosurface_for_vtk->filter->SetInput(grid_for_vtk->GetStructuredGrid());
	isosurface_for_vtk->filter->GenerateValues(5,grid_for_vtk->GetUserRange()->min,grid_for_vtk->GetUserRange()->max);

	isosurface_for_vtk->sinc_filter=vtkWindowedSincPolyDataFilter::New();
	if (isosurface_for_vtk->sinc_filter==NULL) return NULL;
	isosurface_for_vtk->sinc_filter->SetInput(isosurface_for_vtk->filter->GetOutput());
	isosurface_for_vtk->sinc_filter->SetNumberOfIterations(50);

	isosurface_for_vtk->mapper=vtkPolyDataMapper::New();
	if (isosurface_for_vtk->mapper==NULL) return NULL;
	isosurface_for_vtk->mapper->SetInput(isosurface_for_vtk->sinc_filter->GetOutput());
	isosurface_for_vtk->mapper->SetScalarRange(grid_for_vtk->GetUserRange()->min,grid_for_vtk->GetUserRange()->max);
	isosurface_for_vtk->mapper->SetLookupTable(grid_for_vtk->GetLut());

	isosurface_for_vtk->actor=vtkLODActor::New();
	if (isosurface_for_vtk->actor==NULL) return NULL;

	isosurface_for_vtk->actor->SetMapper(isosurface_for_vtk->mapper);
	isosurface_for_vtk->actor->GetProperty()->SetOpacity(1);

	isosurface_for_vtk->grid_for_vtk=grid_for_vtk;
	Define(isosurface_for_vtk);

	return isosurface_for_vtk;
}
