#include <QTcpSocket>
#include <QTimer>
#include <QTextStream>
#include <QTextEdit>

#include <iostream>
#include <strstream>
#include <string>

#include "vteNetwork.h"
#include "vteGUIWindow.h"


/*some POSIX deprecated warning...*/
#define gcvt _gcvt

using namespace std;

/****************************
vteClient::vteClient

Constructor
*****************************/
vteClient::vteClient(long port, const QString &address, const QString &client_name, int connection_speed, vteGUIWindow *parent)
{
	id = -1;
	name = client_name;
	speed = connection_speed;
	parent_window = (vteGUIWindow *)parent;
	connected = false;

	//create the socket and connect various of its signals
	socket = new QTcpSocket( parent );
	connect( socket, SIGNAL(connected()),SLOT(socketConnected()) );
	connect( socket, SIGNAL(connectionClosed()),SLOT(socketConnectionClosed()) );
	connect( socket, SIGNAL(readyRead()),SLOT(socketReadyRead()) );
	connect( socket, SIGNAL(error(QAbstractSocket::SocketError)),SLOT(socketError(QAbstractSocket::SocketError)) );
	connect( this, SIGNAL(ConnectionStatus(int)),parent,SLOT(NetworkConnectionStatus(int)));

	cout<<"Connecting to server"<<endl;
	socket->connectToHost( address, port );
	
	//vteNetworkStatus *stat=new vteNetworkStatus(parent);
	//stat->exec();
}

/*******************************
vteClient::~vteClient

Destructor, closes connection
********************************/
vteClient::~vteClient()
{
	closeConnection();  
}

/**************************
vteClietn::closeConnection

Closes connection
***************************/
void vteClient::closeConnection()
{
	socket->close();
	if ( socket->state() == QTcpSocket::Closing )
      {
            // We have a delayed close.
            connect( socket, SIGNAL(delayedCloseFinished()),SLOT(socketClosed()) );
      } 
	else
      {
            // The socket is closed.
            socketClosed();
      }
}

/********************************
vteClient::socketConnected

Handler of connection signal
We are connected but we still need to validate capVTE server
Send handshake, format is:
	VTE HANDSHAKE protocol_version name chksum1 chksum2
*********************************/
void vteClient::socketConnected()
{
	strstream handshake;
	long int chk1,chk2;

	cout<<socket->state()<<endl;
	cout<<"connection succeeded"<<endl;

	chk1=parent_window->GetVisualizationWindow()->GetInput()->GetCheckSum1();
	chk2=parent_window->GetVisualizationWindow()->GetInput()->GetCheckSum2();
	cout<<"Running network protocol version "<<PROTOCOL_VERSION<<endl;
	handshake<<"VTE CLIENT_HANDSHAKE "<<PROTOCOL_VERSION<<" "<<name.ascii()<<" "<<chk1<<" "<<chk2<<'\x0';

	Send(tr(handshake.str()));
}

/*----------------------------
*/
void vteClient::socketConnectionClosed()
{
	if (connected)
	{
		cout<<"ConnectionClosed"<<endl;
		socket->disconnect();
		connected=false;
	}
}

void vteClient::socketClosed()
{
	cout<<"closed"<<endl;
	delete socket;
}

void vteClient::socketError( QAbstractSocket::SocketError e )
{
	cerr<<"error: "<<e<<endl;
}

void vteClient::Send(const QString &string)
{
	QTextStream ds(socket);
	ds<<string<<endl;
}

/*-----------------------*/
void vteClient::socketReadyRead()
{
	// read from the server
	QTextStream ts( socket );

	while ( socket->canReadLine() )
	{
		QString str=ts.readLine();
		QTextIStream line(&str);

		char command[200];
		line>>command;

		if (stricmp(command,"VTE")==0)
		{
			line>>command;
			
			//handshake response
			if (stricmp(command,"SERVER_HANDSHAKE")==0)
			{
				line>>command;
				if (stricmp(command,"ACCEPT")==0)
				{
					line>>id;
					connected=true;
					emit ConnectionStatus(ACCEPT);
				}
				else if (stricmp(command,"DATA_MISMATCH")==0)
				{                          
					line>>id;
					connected=true;
					emit ConnectionStatus(DATA_MISMATCH);
				}
				else
				{
					emit ConnectionStatus(REJECT);
					connected=false;
				}
			}   //server_handshake
			else
				emit NewData(str);
		}  //VTE

	} //while
}


//---------------------------------
vteClientSocket::vteClientSocket(vteGUIWindow *parent, QTcpSocket *socket, const char *name)
{
	parent_window=parent;

	if (!socket)
		this->socket = new QTcpSocket(parent);
	else
		this->socket = socket;

	connect (this->socket, SIGNAL(readyRead()), SLOT(emitNewDataSignal()));
	socket_id = this->socket->socketDescriptor();
	view=0;
}

vteClientSocket::~vteClientSocket()
{
/*  if (buffer!=NULL) delete(buffer);*/
}

//process client commands
void vteClientSocket::emitNewDataSignal()
{
	QTextStream ts( socket );
	QColor c;
	char command[200],buff[50];
	float version;
	unsigned long chk1, chk2;
  
	while ( socket->canReadLine() )
	{
	    buffer = ts.readLine();
		QTextIStream line(&buffer);

		line>>command;
		if (stricmp(command,"VTE")==0)
		{
			line>>command;
			if (stricmp(command,"CLIENT_HANDSHAKE")==0)
			{
				line>>version;
				if ((float)version!=(float)PROTOCOL_VERSION)
				{
					cout<<"Mismatched network protocol versions, please update!"<<endl;
					cout<<"Check http://www.particleincell.com for the most recent version"<<endl;
					ts<<"VTE SERVER_HANDSHAKE REJECT"<<endl;
					delete this;
				}
				else
				{
					line>>name;
					line>>chk1;
					line>>chk2;
					cout<<"welcome "<<name.latin1()<<endl;
					strcpy(buff,name);
					strcat(buff," is joining the server.");
					c=QColor(200,0,0);
					parent_window->GetNetworkStatus()->textview->setColor(c);
					parent_window->GetNetworkStatus()->textview->append(tr(buff));
					c=QColor(0,0,0);
					parent_window->GetNetworkStatus()->textview->setColor(c);
					strcpy(buff,"VTE CONNECTED ");
					strcat(buff,name);
					parent_window->GetNetwork()->PropagateMessage(buff);

					if ((chk1!=parent_window->GetVisualizationWindow()->GetInput()->GetCheckSum1())||
						(chk2!=parent_window->GetVisualizationWindow()->GetInput()->GetCheckSum2()))
						{
							ts<<"VTE SERVER_HANDSHAKE DATA_MISMATCH "<<socket_id<<endl;
						} 
					else
						{
							ts<<"VTE SERVER_HANDSHAKE ACCEPT "<<socket_id<<endl;
						}
				}
			}   //handshake
			else  
				emit newData(buffer, socket_id);  
		} //vte           
	}
}

/*vteServer----------------------------------------*/
vteServer::vteServer(long port, const QString &name, vteGUIWindow *parent):QTcpServer(parent)
{
	number_of_clients=0;
	parent_window=parent;
	cout<<"Starting server..."<<endl;
	listen(QHostAddress::Any,port);

	if ( isListening() )
	{
		cout<<"Server started successfully..."<<endl;
		parent->GetNetworkStatus()->textview->append(tr("Server started successfully.\n"));
		
		connect( this, SIGNAL(newConnection()),SLOT(AcceptConnection()) );
	}
	else
	{
		qWarning("Failed to start server");
		cout<<errorString().latin1()<<endl;
	}
}

vteServer::~vteServer()
{
}

void vteServer::AcceptConnection()
{
	QTcpSocket *socket = nextPendingConnection();

	client_socket[number_of_clients] = new vteClientSocket( parent_window, socket );
	client_socket[number_of_clients]->view=256;
	client_socket_id[number_of_clients]=socket->socketDescriptor();
	connect(client_socket[number_of_clients]->getSocket(), SIGNAL(newData(QString, int)), SLOT(sendBack(QString, int)));
	emit newConnect(client_socket[number_of_clients]);
	number_of_clients++;
}

void vteServer::sendBack(QString str, int socket_id)
{
	int i,j;
	QString tmp;
	QColor c;
	char command[300],buf[50],message[300],message2[300];

	tmp=str;
	QTextIStream line(&tmp);
	line>>command; // VTE
	line>>command;

	if (stricmp(command,"REMOTEVIEW")==0)
	for (i=0;i<number_of_clients;i++)
	{
		if(client_socket_id[i]==socket_id)
		{
			line>>command;
			client_socket[i]->view=(int)atof(command);
		}
	}
	else if (stricmp(command,"COMMAND")==0)
	{
		line>>command;
		if (stricmp(command,"list")==0)
		{
			for (i=0;i<number_of_clients;i++)
				if(client_socket_id[i]==socket_id)
				{
					QTextStream os(client_socket[i]->getSocket());
					for (j=0;j<number_of_clients;j++)
					{
						strcpy(command,"VTE COMMAND ");
						str=client_socket[j]->GetName();
						strcat(command,str);
						gcvt((float)client_socket_id[j],6,buf);
						strcat(command," ");
						strcat(command,buf);
						str=QString(command);
						os <<str<<endl;
					}
				}
			}
		}
		else if (stricmp(command,"DISCONNECT")==0)
		{
			for (i=0;i<number_of_clients;i++)
			if(client_socket_id[i]==socket_id)
			{
				cout<<"client "<<socket_id<<" disconnected"<<endl;
				strcpy(message,client_socket[i]->GetName());
				strcat(message," has left the server.");
				strcpy(message2,"VTE DISCONNECTED ");
				strcat(message2,client_socket[i]->GetName());
				client_socket[i]=client_socket[number_of_clients];
				number_of_clients--;
				for (j=0;j<number_of_clients;j++)
				{
					if(client_socket_id[j]!=socket_id)
					{
						QTextStream os(client_socket[j]->getSocket());
						os <<QString(message2)<<endl;
					}
				}

				c=QColor(200,0,0);
				parent_window->GetNetworkStatus()->textview->setColor(c);
				parent_window->GetNetworkStatus()->textview->append(message);
				c=QColor(0,0,0);
				parent_window->GetNetworkStatus()->textview->setColor(c);
			}
		}
		else if (stricmp(command,"CONNECTED")==0)
		{
			for (j=0;j<number_of_clients;j++)
			{
				if(client_socket_id[j]!=socket_id)
				{
					QTextStream os(client_socket[j]->getSocket());
					os <<str<<endl;
				}
			}
		}
		else
		{
			for (i=0;i<number_of_clients;i++)
			{
				if(client_socket_id[i]!=socket_id && client_socket[i]->view==socket_id)
				{
					QTextStream os(client_socket[i]->getSocket());
					os <<str<<endl;
				}
			}
		}
}

//vteNetwork-------------------------------------
vteNetwork::vteNetwork(vteGUIWindow *parent, long port, const QString &name, int speed, bool server_flag,const QString &address)
{
	is_server=server_flag;
	parent_window=parent;
 
	if (is_server)
	{
		//start the server
		server=new vteServer(port,name,parent);

		//generate a local client
		QString localhost="localhost";

		client=new vteClient(port,localhost,name,speed,parent);
	}
	else
	{
		server=NULL;
		client=new vteClient(port,address,name,speed,parent);
	}
}


vteNetwork::~vteNetwork()
{
	if (server!=NULL) delete server;
	if (client!=NULL) delete client;
}

int vteNetwork::Disconnect()
{
	delete this;
	return vteOK;
}

int vteNetwork::PropagateMessage(const QString &message, int recipient)
{
	if (recipient==-1) client->Send(message);
	return vteOK;
}

//vteNetworkStatus-------------------------------------
vteNetworkStatus::vteNetworkStatus(vteGUIWindow *parent, const char* name, bool modal, Qt::WFlags fl ): QDialog( parent, name, modal, fl )
{
    //QKeyEvent *e;
    if ( !name )
		setName( "vteNetworkStatus" );

	parent_window=parent;
    //buttongroup = new QButtonGroup(this);
    //buttongroup->setGeometry(QRect(0,200,200,260));

    textview = new QTextEdit( this );
    textview->setReadOnly(true);
    textview->setGeometry( QRect( 0, 0, 250, 200 ) );
	textview->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);

    command_line = new QLineEdit(this);
    command_line->setGeometry(QRect(0,201,200,20));
    //command_line->keyPressEvent(e);

    ok = new QPushButton(this,"ok");
    ok->setGeometry(QRect(210,201,35,20));

    server = new QRadioButton(this,"server");
    server->setGeometry(QRect(20,230,100,30));
    server->setChecked( false );

    other = new QLineEdit(this);
    other->setGeometry(QRect(130,235,30,20));
    other->setEnabled(false);

    apply = new QPushButton(this,"apply");
    apply->setGeometry(QRect(170,235,70,20));
    apply->setEnabled(false);

    disconnect = new QPushButton(this,"disconnect");
    disconnect->setGeometry(QRect(50,260,150,20));
    connect( disconnect, SIGNAL(clicked()), this, SLOT( networkDisconnect()));

    /*self = new QRadioButton(this,"self");
    self->setGeometry(QRect(20,230,100,20));*/
    languageChange();
    connect( server, SIGNAL( toggled(bool) ), other, SLOT( setEnabled(bool) ) );
    connect( server, SIGNAL( toggled(bool) ), apply, SLOT( setEnabled(bool) ) );
    connect( server, SIGNAL( toggled(bool) ), this, SLOT( setRemoteView(bool)));
    connect( apply, SIGNAL( clicked() ),this, SLOT( setRemoteView2()));
    connect( ok, SIGNAL(clicked()), this, SLOT( executeCommand()));

	show();
}

vteNetworkStatus::~vteNetworkStatus()
{
}

void vteNetworkStatus::languageChange()
{
	char message[300];

    sprintf(message,"capVTE Network %2g",PROTOCOL_VERSION);
    setCaption( message );
    server->setText(tr("remote view"));
    apply->setText(tr("Apply"));
    ok->setText(tr("ok"));
    disconnect->setText(tr("Disconnect"));
    //self->setText(tr("self view"));
}

void vteNetworkStatus::setRemoteView(bool remote)
{
	char message[50];
	char text[6];
	int view;

	if(!remote)
		view=256;
	else 
		view=other->text().toInt();

	gcvt((float)view,6,text);

	strcpy(message,"VTE REMOTEVIEW ");
	strcat(message,text);

	parent_window->GetNetwork()->PropagateMessage(message);
}

void vteNetworkStatus::executeCommand()
{
	char message[100];

	strcpy(message,"VTE COMMAND ");
	strcat(message,command_line->text());
	command_line->clear();
	parent_window->GetNetwork()->PropagateMessage(message);
}

void vteNetworkStatus::setRemoteView2()
{
	setRemoteView(true);
}

void vteNetworkStatus::networkDisconnect()
{
	char message[100];

	strcpy(message,"VTE DISCONNECT");

	parent_window->GetNetwork()->PropagateMessage(message);
}
